Package: SCdeconR
Type: Package
Title: Deconvolution of Bulk RNA-Seq Data using Single-Cell RNA-Seq
        Data as Reference
Version: 1.0.2
Authors@R: person("Yuanhang", "Liu", email = "liu.yuanhang@mayo.edu", role = c("aut", "cre"))
Description: Streamlined workflow from deconvolution of bulk RNA-seq data to downstream differential expression and gene-set enrichment analysis. Provide various visualization functions. 
License: GPL (>= 3)
URL: https://github.com/Liuy12/SCdeconR/,
        https://liuy12.github.io/SCdeconR/
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: data.table, gtools, harmony, MASS, Matrix, NMF, reshape2,
        Biobase, mgcv, grDevices, glmGamPoi
Suggests: FARDEEP, reticulate, plotly, DESeq2, knitr, limma, nnls,
        preprocessCore, rmarkdown, scater, scran, SingleCellExperiment,
        doFuture, future, sctransform, Linnorm, spacexr
Depends: dplyr, edgeR, foreach, ggplot2, R (>= 2.10), Seurat
NeedsCompilation: no
Packaged: 2026-04-28 21:59:35 UTC; ext_liu_yuanhang_mayo_edu
Author: Yuanhang Liu [aut, cre]
Maintainer: Yuanhang Liu <liu.yuanhang@mayo.edu>
Repository: CRAN
Date/Publication: 2026-04-29 05:10:02 UTC
Built: R 4.6.0; ; 2026-04-29 06:26:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bulk_generator)
export(celltype_expression)
export(comparedeg_scatter)
export(comparegsea_scatter)
export(compute_metrics)
export(construct_ref)
export(gsea_heatmap)
export(gsea_rwplot)
export(gsea_sumplot)
export(load_scdata)
export(prepare_rnk)
export(prop_barplot)
export(read_gmt)
export(reformat_gmt)
export(run_de)
export(scaling)
export(scdecon)
export(transformation)
export(write_gmt)
import(Seurat)
import(dplyr)
import(edgeR)
import(foreach)
import(ggplot2)
importFrom(MASS,rlm)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(NMF,aheatmap)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(glmGamPoi,glm_gp)
importFrom(glmGamPoi,overdispersion_mle)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rainbow)
importFrom(gtools,mixedsort)
importFrom(mgcv,predict.gam)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)

# SCdeconR

<img src="man/figures/SCdeconR.png" alt="" align="middle" width="100%" height="100%">

SCdeconR aims to provide a streamlined workflow from deconvolution of bulk RNA-seq data to downstream differential and gene-set enrichment analysis. SCdeconR provides a simulation framework to generate artificial bulk samples for benchmarking purposes. It also provides various visualization options to compare the influence of adjusting for cell-proportions differences on differential expression and pathway analyses. 

## Installation

``` r
# install devtools if it's not installed already
if (!require("devtools", quietly = TRUE)) install.packages("devtools")
devtools::install_github("liuy12/SCdeconR")
```

To use [scaden](https://github.com/KevinMenden/scaden) within SCdeconR, follow the below steps:

```r
# install reticulate package first
install.packages("reticulate")
```

Intall scaden python package:

Use pip: 

`pip install scaden`

Or use Conda:

`conda install scaden`

Then provide your desired python path (that have scaden installed) to option `pythonpath` for function `scdecon`. You should be good to go. 

The following packages are optional, and only needed for specific methods within SCdeconR. 

``` r
# install BiocManager if it's not installed already
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")

# data normalization
## scater
BiocManager::install("scater")

## scran
BiocManager::install("scran")

## Linnorm
BiocManager::install("Linnorm")

## SingleCellExperiment
BiocManager::install("SingleCellExperiment")

# deconvolution methods
## FARDEEP
install.packages("FARDEEP")

## nnls
install.packages("nnls")

## MuSiC
devtools::install_github('xuranw/MuSiC')

## SCDC
devtools::install_github("meichendong/SCDC")

# differential expression
## DESeq2
BiocManager::install("DESeq2")

# cell-type specific gene expression
## spacexr
devtools::install_github("dmcable/spacexr", build_vignettes = FALSE)

# interactive plot
install.packages("plotly")
```

## Usage

```r
library(SCdeconR)
```

See [here](https://liuy12.github.io/SCdeconR/) for detailed documentation and tutorials. 

See [here](https://htmlpreview.github.io/?https://github.com/Liuy12/SCdeconR_files/blob/master/inst/reprod_doc/Reproducible_document.html) for a document to reproduce the results from the study.

library(digitalDLSorteR)
library(SingleCellExperiment)

sce <- SingleCellExperiment(assays = ref, 
                            rowData = data.frame(Gene_ID = rownames(ref)),
                            colData = data.frame(Cell_ID = colnames(ref),
                                                 Cell_Type = phenodata$celltype))

bulk_data <- SummarizedExperiment(assays = bulkdata,
                                  rowData = data.frame(Gene_ID = rownames(bulkdata)),
                                  colData = data.frame(Sample_ID = colnames(bulkdata)))

ddls_obj <- createDDLSobject(sc.data = sce,sc.cell.ID.column = "Cell_ID", sc.gene.ID.column = "Gene_ID", sc.cell.type.column = "Cell_Type", bulk.data = bulk_data, bulk.sample.ID.column = "Sample_ID", bulk.gene.ID.column = "Gene_ID")

ddls_obj <- estimateZinbwaveParams(
  object = ddls_obj,
  cell.ID.column = "Cell_ID",
  gene.ID.column = "Gene_ID",
  cell.type.column = "Cell_Type",
  cell.cov.columns = NULL,
  gene.cov.columns = NULL,
  threads = max(1, parallel::detectCores() -1),
  subset.cells = 1000,
  verbose = TRUE
)

#### simulate new sc data
ddls_obj <- simSCProfiles(
  object = ddls_obj,
  cell.ID.column = "Cell_ID",
  cell.type.column = "Cell_Type",
  n.cells = 100,
  suffix.names = "_Simul",
  verbose = TRUE
)

## for reproducibility
set.seed(123)

## prior knowledge for prob.design argument
## need to modify based on your experiment
probMatrix <- data.frame(
  Cell_Type = unique(ddls_obj@single.cell.real$Cell_Type),
  from = c(15,1,15,1,1,15,1,15),
  to = c(70,15,30,15,15,30,15,70)
)

### generate proportion matrix
ddls_obj <- generateBulkCellMatrix(
  object = ddls_obj,
  cell.ID.column = "Cell_ID",
  cell.type.column = "Cell_Type",
  prob.design = probMatrix,
  num.bulk.samples = 1000,
  n.cells = 100,
  verbose = TRUE
)

ddls_obj <- simBulkProfiles(
  object = ddls_obj, type.data = "both", pseudobulk.function = "AddRawCount"
)

reticulate::use_python("/path/to/conda/env/python/")
ddls_obj <- trainDDLSModel(object = ddls_obj, scaling = "standardize")
ddls_obj <- calculateEvalMetrics(object = ddls_obj)

### test on bulk data

ddls_obj <- deconvDDLSObj(
  object = ddls_obj, 
  name.data = "Bulk.DT",
  normalize = TRUE,
  scaling = "standardize",
  verbose = FALSE
)

#' Differential expression analysis
#'
#' Performing differential expression analysis adjusting for cell proportion differences, and other covariates using a additive model.
#'
#' @param bulk a matrix-like object of gene expression values with rows representing genes, columns representing samples
#' @param prop a matrix-like object of cell proportion values with rows representing cell types, columns representing samples. Default to NULL,
#' not adjust for cell proportion.
#' @param sampleinfo a data.frame of metadata for the samples. Rows represents samples; columns represents covariates to adjust for. The first column of
#' sampleinfo should contains group information for differential analysis.
#' @param control a character value indicating the control group in sampleinfo. Set to NULL to perform ANOVA-like analysis.
#' @param case a character value indicating the case group in sampleinfo. Set to NULL to perform ANOVA-like analysis.
#' @param de_method a character value indicating the method to use for testing differential expression. Should be one of "edgeR", "DESeq2", "limma_voom", "limma"
#' @param padj_method method for adjusting multiple hypothesis testing. Default to "BH". See \code{\link{p.adjust}} for more details.
#' @param ... parameters pass to DE methods.
#'
#' @return a list of two elements:
#' \enumerate{
#'  \item a data.frame containing normalized gene expression data.
#'  \item a data.frame containing detailed differential expression statistics. Columns represent "Gene name", "log2 fold change", "log2 average expression",
#' "p value", "adjusted p value" respectively
#' }
#'
#' @details
#' To perform ANOVA like analysis (differences between any groups), set \code{control} & \code{case} options to \code{NULL} and choose one of the following methods:
#' edgeR, limma_voom or limma. DESeq2 does not provide direct support for this type of comparison.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
#'               paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#'
#' # construct integrated reference using harmony algorithm
#' refdata <- construct_ref(ref_list = ref_list,
#'                       phenodata_list = phenodata_list,
#'                       data_type = "cellranger",
#'                       method = "harmony",
#'                       group_var = "subjectid",
#'                       nfeature_rna = 50,
#'                       vars_to_regress = "percent_mt", verbose = FALSE)
#' phenodata <- data.frame(cellid = colnames(refdata),
#'                         celltypes = refdata$celltype,
#'                         subjectid = refdata$subjectid)
#' ## construct a vector with same proportions across cell types
#' prop1 <- data.frame(celltypes = unique(refdata$celltype),
#'                    proportion = rep(0.125, 8))
#' ## simulate 20 bulk samples based on specified cell type proportion
#' bulk_sim1 <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                             phenodata = phenodata,
#'                             num_mixtures = 20,
#'                             prop = prop1, replace = TRUE)
#' ## generate another vector with high proportion for a certian cell type
#' prop2 <- data.frame(celltypes = unique(refdata$celltype),
#'                     proportion = c(0.8, 0.1, 0.1, rep(0, 5)))
#' bulk_sim2 <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                             phenodata = phenodata,
#'                             num_mixtures = 20,
#'                             prop = prop2, replace = TRUE)
#' ## compare data for differential analysis
#' bulk_sim <- list(cbind(bulk_sim1[[1]], bulk_sim2[[1]]),
#'                  cbind(bulk_sim1[[2]], bulk_sim2[[2]]))
#' ## force to be integer for DE purposes
#' bulk <- round(bulk_sim[[1]], digits=0)
#' colnames(bulk) <- paste0("sample", 1:ncol(bulk))
#' ## predict cell type proportions using "OLS" algorithm
#' decon_res <- scdecon(bulk = bulk,
#'                      ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                      phenodata = phenodata,
#'                      filter_ref = TRUE,
#'                      norm_method_sc = "LogNormalize",
#'                      norm_method_bulk = "TMM",
#'                      trans_method_sc = "none",
#'                      trans_method_bulk = "log2",
#'                      marker_strategy = "all")
#' ## create sampleinfo
#' sampleinfo <- data.frame(condition = rep(c("group1", "group2"), each =20))
#' row.names(sampleinfo) <- colnames(bulk)
#' library(DESeq2)
#' deres <- run_de(bulk = bulk,
#'                prop = decon_res[[1]],
#'                sampleinfo = sampleinfo,
#'                control = "group1",
#'                case = "group2",
#'                de_method = "edgeR")
#'
#'  ## run differential analysis without adjusting for cell proportion differences
#'  deres_notadjust <- run_de(bulk = bulk,
#'                            prop = NULL,
#'                            sampleinfo = sampleinfo,
#'                            control = "group1",
#'                            case = "group2",
#'                            de_method = "edgeR")
#'
#' ## scatter plot to compare the effect of adjusting cell proportion differences
#' comparedeg_scatter(results1 = deres[[2]],
#'                    results2 = deres_notadjust[[2]],
#'                    result_names = c("adjust for cell proportion", "not adjust for cell proportion"),
#'                    fc_cutoff = 1.5,
#'                    pval_cutoff = 0.05,
#'                    pvalflag = TRUE,
#'                    interactive = T)
#'
#' ## generate cell-type specific gene expression
#' ct_exprs_list <- celltype_expression(bulk = bulk,
#'                                      ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                                      phenodata = phenodata,
#'                                      prop = decon_res[[1]],
#'                                      UMI_min = 0,
#'                                      CELL_MIN_INSTANCE = 1)
#'
#'
#' }


run_de <- function(
    bulk,
    prop = NULL,
    sampleinfo,
    control = NULL,
    case = NULL,
    de_method = c("edgeR", "DESeq2", "limma_voom", "limma"),
    padj_method = "BH", ...) {
    if (length(intersect(colnames(bulk), rownames(sampleinfo))) != length(union(colnames(bulk), rownames(sampleinfo)))) stop("column names of bulk do not match with row names of sampleinfo.")
    colnames(sampleinfo)[1] <- "group"
    sampleinfo <- sampleinfo[match(colnames(bulk), rownames(sampleinfo)),,drop = FALSE]
    if (!is.null(prop)) {
        if (length(intersect(colnames(bulk), colnames(prop))) != length(union(colnames(bulk), colnames(prop)))) stop("column names of bulk do not match with column names of prop.")
        prop <- prop[, match(colnames(bulk), colnames(prop))]
    }
    if (is.null(control) && (!is.null(case)) || (!is.null(control)) && (is.null(case))) {
        stop("You need to either 1) set values for both control and case, or 2) leave both control and case as NULL for anova-like analysis")
    }
    if ((!is.null(control)) && (!is.null(case)) && ((!control %in% sampleinfo$group) | (!case %in% sampleinfo$group))) {
        stop("control and case defined, but not not found in the first column of sampleinfo")
    }
    if (length(unique(sampleinfo$group)) == 2 && is.null(control) && is.null(case)) {
        stop("Only two conditions found within first column of sampleinfo. You need to set values for both control and case")
    }
    if (de_method == "limma" && !is.null(prop)) stop("limma is recommended for testing cell-type specific gene expression, but prop is provided. Use celltype_expression first.")
    if (!de_method %in% c(c("edgeR", "DESeq2", "limma_voom", "limma"))) stop("de_method has to be one of edgeR, DESeq2, limma_voom or limma")
    if (de_method == "edgeR") {
        results <- edger_fun(bulk, prop, sampleinfo, control, case, padj_method, ...)
    } else if (de_method == "DESeq2") {
        if (length(unique(sampleinfo$group)) > 2) stop("DE analysis with group > 2 is not directly supported by DESeq2") else results <- deseq2_fun(bulk, prop, sampleinfo, control, case, padj_method, ...)
    } else if (de_method == "limma_voom"){
        results <- limma_voom_fun(bulk, prop, sampleinfo, control, case, padj_method, ...)
    } else {
        results <- limma_fun(bulk, sampleinfo, control, case, padj_method, ...)
    }
}


#' Generate a scatter plot comparing two differential expression results
#'
#' Generate a scatter plot of fold changes comparing two differential expression results, e.g. w/wo adjusting for cell proportion differences.
#'
#' @param results1 a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
#' "p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.
#' @param results2 similar to \code{results1}.
#' @param result_names a vector of length 2 indicating the names of the two differential results. If NULL, names will be set to c("results1", "results2")
#' @param fc_cutoff fold change cutoff to identify differential expressed genes.
#' @param pval_cutoff p value cutoff to identify differential expressed genes.
#' @param pvalflag a logical value indicating whether to use adjusted p value in selecting differential expressed genes.
#' @param interactive a logical value indicating whether to generate an interactive plot.
#'
#' @details See examples from \code{\link{run_de}}.
#' @export
#'
#'

comparedeg_scatter <- function(
    results1,
    results2,
    result_names = NULL,
    fc_cutoff,
    pval_cutoff,
    pvalflag = TRUE,
    interactive = FALSE) {
    if (ncol(results1) != 5 || ncol(results2) != 5) stop("inputs need to have five columns: genename, log2foldchange, log2avgexp, pval & padj")
    colnames(results1) <- colnames(results2) <- c("genename", "log2foldchange", "log2avgexp", "pval", "padj")
    if ((!is.null(result_names)) && length(result_names) != 2) stop("The length of result_names has to equal to 2") else if (is.null(result_names)) result_names <- c("result1", "result2")
    results_combined <- inner_join(results1, results2, by = "genename")
    if (pvalflag) {
        idx1 <- which(abs(results_combined$log2foldchange.x) >= log2(fc_cutoff) & results_combined$padj.x <= pval_cutoff)
        idx2 <- which(abs(results_combined$log2foldchange.y) >= log2(fc_cutoff) & results_combined$padj.y <= pval_cutoff)
    } else {
        idx1 <- which(abs(results_combined$log2foldchange.x) >= log2(fc_cutoff) & results_combined$pval.x <= pval_cutoff)
        idx2 <- which(abs(results_combined$log2foldchange.y) >= log2(fc_cutoff) & results_combined$pval.y <= pval_cutoff)
    }
    if (length(idx1) == 0 && length(idx2) == 0) stop("No DEGs identified, try loosing the thresholds")
    results_combined$category <- "Not significant"
    results_combined$category[intersect(idx1, idx2)] <- "Both"
    results_combined$category[setdiff(idx1, idx2)] <- paste0(result_names[1], " only")
    results_combined$category[setdiff(idx2, idx1)] <- paste0(result_names[2], " only")
    color_df <- data.frame(
        group = c("Both", paste0(result_names[1], " only"), paste0(result_names[2], " only"), "Not significant"),
        colors = c("red", "seagreen1", "royalblue", "grey")
    )
    results_combined$category <- factor(results_combined$category, levels = color_df$group[color_df$group %in% unique(results_combined$category)])
    lim_values <- range(results_combined[, c("log2foldchange.x", "log2foldchange.y")], na.rm = TRUE)
    gp <- ggplot() +
        geom_point(aes(x = log2foldchange.x, y = log2foldchange.y, color = category, text = paste0("GeneSymbol: ", genename)), size = 5, data = results_combined) +
        theme_classic() + labs(x= result_names[1], y = result_names[2]) +
        scale_color_manual(values = color_df$colors[color_df$group %in% unique(results_combined$category)]) +
        coord_cartesian(xlim = lim_values, ylim = lim_values) +
        geom_hline(yintercept = c(log2(fc_cutoff), -log2(fc_cutoff)), linetype = 2) +
        geom_vline(xintercept = c(log2(fc_cutoff), -log2(fc_cutoff)), linetype = 2) +
        geom_abline(slope = 1, intercept = 0)
    if (interactive) {
        return(plotly::ggplotly(gp))
    } else {
        return(gp)
    }
}

#' Compute cell type specific gene expression
#'
#' Compute cell type specific gene expression based on predicted cell proportions and reference data.
#'
#' @param bulk a matrix-like object of bulk RNA-seq data with rows representing genes, columns representing samples
#' @param ref a matrix-like object of scRNA-seq data with rows representing genes, columns representing cells.
#' @param phenodata a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first two
#' columns as:
#' \enumerate{
#'  \item cell barcodes
#'  \item cell types
#' }
#' @param prop a matrix-like object of cell proportion values with rows representing cell types, columns representing samples.
#' @param ... additional parameters passed to \code{create.RCTD} from \code{spacexr}.
#'
#' @return a list with length equal to number of unique cell types in phenodata. Each element in the list represents gene expression matrix for each unique cell type.
#'
#' @details this function is inspired by cell-type specific gene expression estimation for doublet mode in \code{spacexr}. See examples from \code{\link{run_de}}.
#'
#' @export
#'

celltype_expression <- function(bulk, ref, phenodata, prop, ...) {
    if (ncol(phenodata) < 2) stop("phenodata should contain at least the first two columns: cellid, celltype.")
    colnames(phenodata)[1:2] <- c("cellid", "celltype")
    if (length(unique(phenodata$cellid)) != nrow(phenodata)) stop("values of cellid in phenodata not unique.")
    if (length(intersect(colnames(ref), phenodata$cellid)) != length(union(colnames(ref), phenodata$cellid))) stop("column names of reference data do not match with cellid of the phenodata.")
    if (length(union(colnames(bulk), colnames(prop))) != length(intersect(colnames(bulk), colnames(prop)))) stop("observed inconsistency between column names of bulk and prop")
    prop <- prop[, match(colnames(bulk), colnames(prop))]
    phenodata <- phenodata[match(colnames(ref), phenodata$cellid), ]
    celltypes <- factor(phenodata[, "celltype"])
    names(celltypes) <- phenodata$cellid
    ### use functions from spacexr package
    reference <- eval(parse(text = 'spacexr::Reference(ref, celltypes, require_int = FALSE, min_UMI = 0)'))
    bulk_fake_spatial <- eval(parse(text = 'spacexr::SpatialRNA(counts = bulk, use_fake_coords = TRUE, require_int = FALSE)'))
    rctd_obj <- eval(parse(text = 'spacexr::create.RCTD(bulk_fake_spatial, reference, fc_cutoff = 0, ...)'))
    rctd_obj <- eval(parse(text = 'spacexr::fitBulk(rctd_obj)'))
    celltypes <- rctd_obj@cell_type_info$info[[2]]
    ct_renorm <- rctd_obj@cell_type_info$renorm[[1]]
    ct_mat <- lapply(1:length(celltypes), function(i) {
        mat1 <- matrix(ct_renorm[, colnames(ct_renorm) == celltypes[i]], nrow = nrow(ct_renorm), ncol = 1)
        mat2 <- matrix(prop[rownames(prop) == celltypes[i], ], nrow = 1, ncol = ncol(prop))
        mat1 %*% mat2
    })
    ct_sum <- Reduce("+", ct_mat)
    ct_exp <- vector("list", length(celltypes))
    names(ct_exp) <- celltypes
    for (i in 1:length(celltypes)) ct_exp[[i]] <- rctd_obj@spatialRNA@counts * (ct_mat[[i]] / ct_sum)
    return(ct_exp)
}


deseq2_fun <- function(counts, prop, sampleinfo, control, case, padj_method, ...) {
    message("Carrying out differential expression analysis using DESeq2!")
    if (!is.null(prop)) {
        ## remove one cell-type to make design matrix full rank
        prop <- prop[-which.min(apply(prop, 1, median)), ]
        sampleinfo <- cbind(sampleinfo, t(prop))
    }
    colnames(sampleinfo) <- make.names(colnames(sampleinfo))
    formula_de <- as.formula(paste0("~ ", paste0(colnames(sampleinfo), collapse = "+")))
    dse <- DESeq2::DESeqDataSetFromMatrix(countData = counts, colData = sampleinfo, design = formula_de)
    dse <- DESeq2::estimateSizeFactors(dse)
    normdata <- DESeq2::counts(dse, normalized = TRUE)
    dse <- DESeq2::estimateDispersions(dse, ...)
    dse <- DESeq2::nbinomWaldTest(dse)
    res <- DESeq2::results(dse, cooksCutoff = FALSE, contrast = c("group", case, control))
    res <- as.data.frame(res)
    teststats <- res %>%
        select(-c(lfcSE, stat, padj)) %>%
        dplyr::rename(log2foldchange = log2FoldChange, pval = pvalue) %>%
        mutate(log2avgexp = log2(baseMean + 0.1), .after = log2foldchange) %>%
        mutate(padj = p.adjust(pval, method = padj_method)) %>%
        mutate(genename = rownames(res), .before = log2foldchange) %>%
        select(-baseMean)
    list(normdata, teststats)
}


edger_fun <- function(counts, prop, sampleinfo, control, case, padj_method, ...) {
    message("Carrying out differential expression analysis using edgeR!")
    cds <- DGEList(counts = counts, group = sampleinfo$group)
    cds <- calcNormFactors(cds)
    if (!is.null(prop)) {
        ## remove one cell-type to make design matrix full rank
        prop <- prop[-which.min(apply(prop, 1, median)), ]
        sampleinfo <- cbind(sampleinfo, t(prop))
    }
    colnames(sampleinfo) <- make.names(colnames(sampleinfo))
    if (is.null(control) && is.null(case)) {
        formula_de <- as.formula(paste0("~ ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[2:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[2:length(unique(sampleinfo$group))])
        cds <- estimateGLMRobustDisp(cds, design, ...)
        fit <- glmQLFit(cds, design)
        lrt <- glmQLFTest(fit, coef = 2:length(unique(sampleinfo$group)))
    } else {
        formula_de <- as.formula(paste0("~ 0 + ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[1:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[1:length(unique(sampleinfo$group))])
        cds <- estimateGLMRobustDisp(cds, design, ...)
        fit <- glmQLFit(cds, design)
        lrt <- eval(parse(text = paste0("glmQLFTest(fit,contrast=makeContrasts(",case,"-",control,",levels=design))")))
    }
    normdata <- cpm(cds)
    teststats <- lrt$table %>%
        select(-F) %>%
        dplyr::rename(logavgexp = logCPM, pval = PValue) %>%
        rename_with(~ gsub("logFC", "log2foldchange", .x)) %>%
        mutate(padj = p.adjust(pval, method = padj_method)) %>%
        mutate(genename = rownames(lrt$table), .before = log2foldchange)
    return(list(normdata, teststats))
}


limma_voom_fun <- function(counts, prop, sampleinfo, control, case, padj_method, ...) {
    message(paste0("Carrying out differential expression analysis using limma_voom!", "\n"))
    if (!is.null(prop)) {
        ## remove one cell-type to make design matrix full rank
        prop <- prop[-which.min(apply(prop, 1, median)), ]
        sampleinfo <- cbind(sampleinfo, t(prop))
    }
    colnames(sampleinfo) <- make.names(colnames(sampleinfo))
    nf <- calcNormFactors(counts)
    y <- limma::voom(counts, plot = FALSE, lib.size = colSums(counts) * nf)
    normdata <- as.matrix(2^(y$E))
    if (is.null(control) && is.null(case)) {
        formula_de <- as.formula(paste0("~ ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[2:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[2:length(unique(sampleinfo$group))])
        fit <- limma::lmFit(y, design, ...)
        fit <- limma::eBayes(fit)
        lmres <- limma::topTable(fit, coef = 2:length(unique(sampleinfo$group)), n = nrow(counts), sort.by = "none")
        ncoef <- length(unique(sampleinfo$group)) - 1
    } else {
        formula_de <- as.formula(paste0("~ 0 + ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[1:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[1:length(unique(sampleinfo$group))])
        fit <- limma::lmFit(y, design, ...)
        fit2 <- eval(parse(text = paste0("limma::contrasts.fit(fit, contrasts = limma::makeContrasts(", case,"-",control,",levels=design))")))
        fit2 <- limma::eBayes(fit2)
        lmres <- limma::topTable(fit2, coef = 1, n = nrow(counts), sort.by = "none")
        ncoef <- 1
    }
    teststats <- lmres %>%
        select(-c(F, adj.P.Val)) %>%
        dplyr::rename(pval = P.Value, log2avgexp = AveExpr) %>%
        rename_with(~ gsub("^", "log2foldchange.", .x), .cols = 1:all_of(ncoef)) %>%
        mutate(padj = p.adjust(pval, method = padj_method)) %>%
        mutate(genename = rownames(lmres), .before = log2foldchange)
    return(list(normdata, teststats))
}


limma_fun <- function(counts, prop, sampleinfo, control, case, p_adj_method, ...) {
    message(paste0("Carrying out differential expression analysis using limma!", "\n"))
    normdata <- preprocessCore::normalize.quantiles(as.matrix(counts))
    rownames(normdata) <- rownames(counts)
    colnames(normdata) <- colnames(counts)
    normdata_log2 <- log2(normdata + 0.01)
    colnames(sampleinfo) <- make.names(colnames(sampleinfo))
    if (is.null(control) && is.null(case)) {
        formula_de <- as.formula(paste0("~ ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[2:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[2:length(unique(sampleinfo$group))])
        fit <- limma::lmFit(normdata_log2, design, ...)
        fit <- limma::eBayes(fit)
        lmres <- limma::topTable(fit, coef = 2:length(unique(sampleinfo$group)), n = nrow(counts), sort.by = "none")
        ncoef <- length(unique(sampleinfo$group)) - 1
    } else {
        formula_de <- as.formula(paste0("~ 0 + ", paste0(colnames(sampleinfo), collapse = "+")))
        design <- model.matrix(formula_de, data = sampleinfo)
        colnames(design)[1:length(unique(sampleinfo$group))] <- gsub("^group", "", colnames(design)[1:length(unique(sampleinfo$group))])
        fit <- limma::lmFit(normdata_log2, design, ...)
        fit2 <- eval(parse(text = paste0("limma::contrasts.fit(fit, contrasts = limma::makeContrasts(", case,"-",control,",levels=design))")))
        fit2 <- limma::eBayes(fit2)
        lmres <- limma::topTable(fit2, coef = 1, n = nrow(counts), sort.by = "none")
        ncoef <- 1
    }
    teststats <- lmres %>%
        select(-c(F, adj.P.Val)) %>%
        dplyr::rename(pval = P.Value, log2avgexp = AveExpr) %>%
        rename_with(~ gsub("^", "log2foldchange.", .x), .cols = 1:all_of(ncoef)) %>%
        mutate(padj = p.adjust(pval, method = padj_method)) %>%
        mutate(genename = rownames(lmres), .before = log2foldchange)
    return(list(normdata, teststats))
}

#' Integration of single-cell/nuclei RNA-seq data as reference
#'
#' Integration of scRNA-seq or snRNA-seq data using either harmony or seurat.
#'
#' @param ref_list a character vector of data paths to scRNA-seq/snRNA-seq. See \code{data_type} for accepted data types.
#' @param phenodata_list a character vector of data paths to metadata for elements in ref_list. All metadata within phenodata_list should have consistent column names. Columns represent
#' cell attributes, such as cell type, rows represent cells. Each element in \code{phenodata_list} should at least contain the first two columns as:
#' \enumerate{
#'  \item cell barcodes
#'  \item cell types
#' }
#' @param data_type data type of the input scRNA-seq/snRNA-seq data. Could be either a single character value from "cellranger", "h5", "matrix", or
#' a vector/list of values with the same length as ref_list indicating the data type for each element.
#' @param method character value specifying the method to use. Has to be one of "harmony" or "seurat". See details for more information.
#' @param group_var a vector of character values indicating which variables within phenodata_list metadata to use for integration. Only applicable when method
#' is set to "harmony".
#' @param nfeature_rna minimum # of features with non-zero UMIs. Cells with # of features lower than nfeature_rna will be removed. Default to 200.
#' @param percent_mt maximum percentage of mitochondria (MT) mapped UMIs. Cells with MT percentage higher than percent_mt will be removed. Default to 40.
#' @param vars_to_regress a list of character values indicating the variables to regress for SCTransform normalization step. Default is to regress
#' out MT percentage ("percent_mt") & cell cycle effects ("phase")
#' @param ex_features a vector of character values indicating genes to exclude from anchor features. Those genes will not be considered as
#' anchor features for integration, but will still be present in the integrated data.
#' @param cluster logical value indicating whether to perform clustering on the integrated data. If TRUE, unsupervised clustering will be performed, and
#' the results will be saved in "seurat_clusters" metadata in the output Seurat object.
#' @param resolution numeric value specifying resolution to use when cluster is set to TRUE.
#' @param verbose logical value indicating whether to print messages.
#' @param ... additional parameters passed to \code{\link[Seurat]{SCTransform}}.
#'
#' @return a \code{\link[Seurat]{Seurat-class}} object.
#'
#' @details
#' data_type can be chosen from:
#' \describe{
#'  \item{cellranger}{path to a directory containing the matrix.mtx, genes.tsv (or features.tsv), and barcodes.tsv files outputted by 10x's cell-ranger}
#'  \item{h5}{path to .h5 file outputted by 10x's cell-ranger}
#'  \item{matrix}{path to a matrix-like file, with rows representing genes, columns representing cells.}
#' }
#'
#' SCTransform with \code{vst.flavor = "v2"} is used for normalization of individual data. Integration methods can be chosen from either "harmony"
#' or "seurat". Harmony typically is more memory efficient and, recommended if you have large # of cells for integration.
#'
#' @export
#'
#' @examples
#' \donttest{
#' ## random subset of two scRNA-seq datasets for breast tissue
#' ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
#'               paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#'
#' ## Register backend for parallel processing
#' #registerDoFuture()
#' #plan("multisession", workers = 4)
#'
#' ## construct integrated reference data
#' refdata <- construct_ref(ref_list = ref_list,
#'                          phenodata_list = phenodata_list,
#'                          data_type = "cellranger",
#'                          method = "harmony",
#'                          group_var = "subjectid",
#'                          nfeature_rna = 50,
#'                          vars_to_regress = "percent_mt")
#' }


construct_ref <- function(
    ref_list,
    phenodata_list,
    data_type = c("cellranger", "h5", "matrix"),
    method = c("harmony", "seurat"),
    group_var,
    nfeature_rna = 200,
    percent_mt = 40,
    vars_to_regress = c("percent_mt", "phase"),
    ex_features = NULL,
    cluster = TRUE,
    resolution = 0.8,
    verbose = TRUE, ...) {
  phenodata_list <- lapply(phenodata_list, function(i) data.table::fread(file = i, check.names = FALSE, header = TRUE, data.table = FALSE))
  phenodata_list_names <- lapply(phenodata_list, function(i) colnames(i))
  if (any(Reduce(intersect, phenodata_list_names) != phenodata_list_names[[which.max(sapply(phenodata_list_names, function(i) length(i)))]])) {
    stop("phenodata_list contains inconsistent colnames")
  }
  if (!group_var %in% colnames(phenodata_list[[1]])) {
    stop(paste0("groupvar not found in column names of phenodata_list: "), paste0(phenodata_list_names[[1]], collapse = ","))
  }
  if (!all(sapply(data_type, function(i) i %in% c("cellranger", "h5", "matrix")))) stop(paste0("data_type has to be one of: ", paste0(c("cellranger", "h5", "matrix"), collapse = ",")))
  if (length(data_type) > 1 && length(data_type) != length(ref_list)) stop("data_type could be either a single character value from 'cellranger', 'h5', 'matrix', or a vector of values with the same length as ref_list indicating the data type for each element")
  if (length(data_type) ==1) data_type <- rep(data_type, times = length(ref_list))
  object_list <- foreach(i = seq_along(ref_list), .packages = "Seurat") %dopar%
    load_scdata(
      ref = ref_list[[i]],
      data_type = data_type[[i]],
      meta_info = phenodata_list[[i]],
      nfeature_rna = nfeature_rna,
      percent_mt = percent_mt,
      vars_to_regress = vars_to_regress,
      id = i,
      verbose = verbose, ...
    )
  if (verbose) message("select integration features")
  object_features <- SelectIntegrationFeatures(object.list = object_list, nfeatures = 3000, verbose = verbose)
  object_features <- setdiff(object_features, ex_features)
  if (method == "seurat") {
    object_list <- PrepSCTIntegration(object.list = object_list, anchor.features = object_features, verbose = verbose, assay = "SCT")
    merge_anchor <- FindIntegrationAnchors(object.list = object_list, dims = 1:30, normalization.method = "SCT", anchor.features = object_features, verbose = verbose)
    rm(list = c("object_list"))
    gc(verbose = verbose)
    message("perform integration")
    merge_data <- IntegrateData(anchorset = merge_anchor, normalization.method = "SCT", verbose = verbose, dims = 1:30)
    rm(list = c("merge_anchor"))
    gc(verbose = verbose)
  } else if (method == "harmony") {
    if (verbose) message("Generating merged data for harmony")
    merge_data <- merge(object_list[[1]], y = object_list[2:length(object_list)], merge.data = TRUE)
    rm(list = c("object_list"))
    gc(verbose = verbose)
    VariableFeatures(merge_data) <- object_features
    merge_data <- RunPCA(object = merge_data, assay = "SCT", verbose = verbose, npcs = 30)
    if (verbose) message("Running harmony")
    merge_data <- harmony::RunHarmony(
      object = merge_data,
      reduction.use = "pca",
      dims.use = 1:30,
      group.by.vars = group_var,
      plot_convergence = FALSE,
      verbose = verbose
    )
  }
  if (cluster) {
    reduction <- ifelse(method == "harmony", "harmony", "pca")
    merge_data <- RunPCA(merge_data, npcs = 30, verbose = verbose, features = VariableFeatures(merge_data))
    merge_data <- RunTSNE(merge_data, reduction = reduction, dims = 1:30, verbose = verbose)
    merge_data <- RunUMAP(merge_data, reduction = reduction, dims = 1:30, verbose = verbose)
    merge_data <- FindNeighbors(merge_data, reduction = reduction, dims = 1:30, verbose = verbose)
    merge_data <- FindClusters(merge_data, resolution = resolution, verbose = verbose)
  }
  if (verbose) message("normalizing median UMI counts across SCT models")
  merge_data <- PrepSCTFindMarkers(merge_data, verbose = verbose)
  return(merge_data)
}

#' Load, filter and normalize scRNA-seq/snRNA-seq data
#'
#' Load and preprocess scRNA-seq/snRNA-seq data using seurat SCTransform workflow.
#'
#' @param ref path to scRNA-seq/snRNA-seq data.
#' @param data_type a character value specifying data type of the input scRNA-seq/snRNA-seq data, should be one of "cellranger", "h5", "matrix".
#' @param meta_info a data.frame with rows representing cells, columns representing cell attributes.
#' @param nfeature_rna minimum # of features with non-zero UMIs. Cells with # of features lower than nfeature_rna will be removed. Default to 200.
#' @param percent_mt maximum percentage of mitochondria (MT) mapped UMIs. Cells with MT percentage higher than percent_mt will be removed. Default to 40.
#' @param cc.genes cell-cycle genes curated by Seurat. Can be loaded via \code{data(cc.genes)}
#' @param vars_to_regress a list of character values indicating the variables to regress for SCTransform normalization step. Default is to regress
#' out MT percentage ("percent_mt") & cell cycle effects ("phase")
#' @param id a character value specifying project or sample id. Only used for printing purposes.
#' @param verbose logical value indicating whether to print messages.
#' @param ... additional parameters passed to \code{\link[Seurat]{SCTransform}}.
#'
#' @return a \code{\link[Seurat]{Seurat-class}} object.
#'
#' @details
#' For more details, refer to \code{\link{construct_ref}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' samplepath1 <- paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1")
#' samplepath2 <- paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2")
#' ref_list <- c(samplepath1, samplepath2)
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#' tmp <- load_scdata(
#'   ref = ref_list[[1]],
#'   data_type = c("cellranger"),
#'   meta_info = data.table::fread(file = phenodata_list[[1]], check.names = FALSE, header = TRUE),
#'   nfeature_rna = 50,
#'   vars_to_regress = c("percent_mt"),
#'   id = 1,
#'   verbose = TRUE)
#' }


load_scdata <- function(
    ref,
    data_type = c("cellranger", "h5", "matrix"),
    meta_info,
    nfeature_rna = 200,
    percent_mt = 40,
    cc.genes = NULL,
    vars_to_regress = c("percent_mt", "phase"),
    id,
    verbose, ...) {
  if (data_type == "cellranger") {
    data <- Read10X(data.dir = ref)
  } else if (data_type == "h5") {
    data <- Read10X_h5(filename = ref)
  } else {
    data <- data.table::fread(file = ref, data.table = FALSE)
  }
  if (verbose) message(paste0("Loading sample completed for sample ", id))
  rownames(meta_info) <- meta_info[[1]]
  if (length(intersect(rownames(meta_info), colnames(data))) != length(union(rownames(meta_info), colnames(data)))) {
    stop(paste0("cell barcodes mismatch betweeen expression data and meta data for sample ", id))
  }
  meta_info <- meta_info[match(colnames(data), rownames(meta_info)), ]
  if (verbose) message(paste0("create seurat object for sample ", id))
  data <- CreateSeuratObject(counts = data, meta.data = as.data.frame(meta_info))
  if (verbose) message(paste0("calculate mt percent for sample "), id)
  data[["percent_mt"]] <- PercentageFeatureSet(object = data, pattern = "^MT-|^mt-")
  cellids <- colnames(data)[data$nFeature_RNA >= nfeature_rna & data$percent_mt <= percent_mt]
  data <- subset(data, cells = cellids)
  if ("phase" %in% vars_to_regress) {
    if (verbose) message(paste0("predict cell cycle phase for sample ", id))
    data <- NormalizeData(data, assay = "RNA", verbose = verbose)
    #data("cc.genes")
    data <- CellCycleScoring(data, s.features = cc.genes$s.genes, g2m.features = cc.genes$g2m.genes, set.ident = FALSE, verbose = verbose)
  }
  if (verbose) message(paste0("perform sctransform for sample ", id))
  data <- SCTransform(data, variable.features.n = 3000, vst.flavor = "v2", vars.to.regress = vars_to_regress, verbose = verbose, return.only.var.genes = FALSE, ...)
  return(data)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import Seurat
#' @import dplyr
#' @import edgeR
#' @import foreach
#' @import ggplot2
#' @importFrom MASS rlm
#' @importFrom Matrix colSums
#' @importFrom Matrix rowMeans
#' @importFrom Matrix rowSums
#' @importFrom NMF aheatmap
#' @importFrom data.table fread
#' @importFrom data.table fwrite
#' @importFrom glmGamPoi glm_gp
#' @importFrom glmGamPoi overdispersion_mle
#' @importFrom grDevices colorRampPalette
#' @importFrom grDevices rainbow
#' @importFrom gtools mixedsort
#' @importFrom mgcv predict.gam
#' @importFrom reshape2 dcast
#' @importFrom reshape2 melt
## usethis namespace: end
NULL

filter_cells <- function(filter_param) {
  cellstoremove <- which(filter_param > median(filter_param) + 3 * mad(filter_param) | filter_param < median(filter_param) - 3 * mad(filter_param))
  cellstoremove
}

Normalization <- function(data) {
  data <- DGEList(data)
  CtrlGenes <- grep("ERCC-", rownames(data))

  if (length(CtrlGenes) > 1) {
    spikes <- data[CtrlGenes, ]
    spikes <- calcNormFactors(spikes, method = "TMM")
    data$samples$norm.factors <- spikes$samples$norm.factors
  } else {
    data <- calcNormFactors(data, method = "TMM")
  }

  return(data)
}

markerfc <- function(fit2, log2_threshold = log2(1.5), output_name = "markers") {
  topTable_RESULTS <- limma::topTable(fit2, coef = 1:ncol(fit2$contrasts), number = Inf, adjust.method = "BH", p.value = 0.05, lfc = log2_threshold)
  AveExpr_pval <- topTable_RESULTS[, (ncol(topTable_RESULTS) - 3):ncol(topTable_RESULTS)]
  topTable_RESULTS <- topTable_RESULTS[, 1:(ncol(topTable_RESULTS) - 4)]

  if (length(grep("ERCC-", topTable_RESULTS$gene)) > 0) {
    topTable_RESULTS <- topTable_RESULTS[-grep("ERCC-", topTable_RESULTS$gene), ]
  }

  markers <- apply(topTable_RESULTS, 1, function(x) {
    temp <- sort(x)
    ((temp[ncol(topTable_RESULTS)] - temp[ncol(topTable_RESULTS) - 1]) >= log2_threshold) | (abs(temp[1] - temp[2]) >= log2_threshold)
  })

  topTable_RESULTS <- topTable_RESULTS[markers, ]

  markers <- cbind.data.frame(
    rownames(topTable_RESULTS),
    t(apply(topTable_RESULTS, 1, function(x) {
      temp <- max(x)
      if (temp < log2_threshold) {
        temp <- c(min(x), colnames(topTable_RESULTS)[which.min(x)])
      } else {
        temp <- c(max(x), colnames(topTable_RESULTS)[which.max(x)])
      }
      temp
    }))
  )

  colnames(markers) <- c("gene", "log2FC", "CT")
  markers$log2FC <- as.numeric(as.character(markers$log2FC))
  markers <- markers %>% arrange(CT, desc(log2FC))

  markers$AveExpr <- AveExpr_pval$AveExpr[match(markers$gene, rownames(AveExpr_pval))]
  markers$gene <- as.character(markers$gene)
  markers$CT <- as.character(markers$CT)

  return(markers)
}



marker_strategies <- function(marker_distrib, marker_strategy) {
  if (marker_strategy == "all") {

    # using all markers that were found
    markers <- marker_distrib
  } else if (marker_strategy == "pos_fc") {

    # using only markers with positive FC (=over-expressed in cell type of interest)
    markers <- marker_distrib %>%
      filter(log2FC > 0) %>%
      as.data.frame()
  } else if (marker_strategy == "top_50p_logFC") {

    # top 50% of markers (per CT) based on logFC
    markers <- marker_distrib %>%
      filter(log2FC > 0) %>%
      arrange(CT, desc(log2FC)) %>%
      group_by(CT) %>%
      top_n(ceiling(n() * 0.5), wt = log2FC) %>%
      as.data.frame()
  } else if (marker_strategy == "top_50p_AveExpr") {

    # top 50% of markers based on average gene expression (baseline expression)
    markers <- marker_distrib %>%
      filter(log2FC > 0) %>%
      arrange(CT, desc(AveExpr)) %>%
      group_by(CT) %>%
      top_n(ceiling(n() * 0.5), wt = log2FC) %>%
      as.data.frame()
  }

  return(markers)
}

#' Transformation of gene expression data
#'
#' Methods to use for data transformation.
#'
#' @param matrix a matrix-like objector of gene expression values with rows representing genes, columns representing samples or cells
#' @param option character value specifying the transformation method to use. Has to be one of "none", "log", "sqrt", "vst".
#' @return a matrix-like object with the same dimension of input object after data transformation.
#'
#' @details
#' refer to \code{\link{scdecon}} for more details.
#' @export


transformation <- function(matrix, option) {

  #############################################################
  ##########    DATA TRANSFORMATION (on full data)   ##########
  if (option == "none") {
    matrix <- matrix
  }

  if (option == "log2") {
    matrix <- log2(matrix + 1)
  }

  if (option == "sqrt") {
    matrix <- sqrt(matrix)
  }

  if (option == "vst") {
    matrix <- DESeq2::varianceStabilizingTransformation(as.matrix(matrix))
  }

  return(matrix)
}


#' Normalization of gene expression data
#'
#' Methods to use for data normalization.
#'
#' @param matrix a matrix-like objector of gene expression values with rows representing genes, columns representing samples or cells
#' @param option character value specifying the normalization method to use. Has to be one of "none", "LogNormalize", "TMM", "median_ratios", "TPM",
#' "SCTransform", "scran", "scater", "Linnorm".
#' @param gene_length a data.frame with two columns. The first column represents gene names that match with provided bulk data. The second column
#' represents length of each gene. Only applicable when norm_method is selected as "TPM"
#' @param seed random seed used for simulating FFPE artifacts. Only applicable when ffpe_artifacts is set to TRUE.
#' @param ffpe_artifacts logical value indicating whether to add simulated ffpe artifacts in the bulk data. Only applicable to simulation experiments in
#' evaluating the effect of FFPE artifacts.
#' @param model pre-constructed ffpe model data. Can be downloaded from github: https://github.com/Liuy12/SCdeconR_files/blob/master/data/ffpemodel.rda
#'
#' @return a matrix-like object with the same dimension of input object after data normalization.
#'
#' @details
#' refer to \code{\link{scdecon}} for more details.
#' @export


scaling <- function(matrix, option, gene_length = NULL, seed = 1234, ffpe_artifacts = FALSE, model = NULL) {

  ##########    Remove rows & columns full of zeroes   ##########
  matrix <- matrix[rowSums(matrix) != 0, ]
  # OLS with error if all elements within a row are equal (e.g. all 0, or all a common value after log/sqrt/vst transformation)
  matrix <- matrix[!apply(matrix, 1, function(x) var(x) == 0), ]
  matrix <- matrix[, colSums(matrix) != 0]

  if (option == "LogNormalize") {
    matrix <- as.matrix(expm1(LogNormalize(matrix, verbose = FALSE))) # for v3
  } else if (option == "TMM") {
    matrix <- DGEList(counts = matrix)
    CtrlGenes <- grep("ERCC-", rownames(data))

    if (length(CtrlGenes) > 1) {
      spikes <- data[CtrlGenes, ]
      spikes <- calcNormFactors(spikes, method = "TMM")
      matrix$samples$norm.factors <- spikes$samples$norm.factors
    } else {
      matrix <- calcNormFactors(matrix, method = "TMM")
    }

    matrix <- cpm(matrix)
  } else if (option == "median_ratios") { # requires integer values
    metadata <- data.frame(sampleid = colnames(matrix))
    CtrlGenes <- grep("ERCC-", rownames(matrix))
    matrix <- DESeq2::DESeqDataSetFromMatrix(matrix, colData = metadata, design = ~1)
    if (length(CtrlGenes) > 1 & sum(rowSums(counts(matrix[CtrlGenes, ]) != 0) >= 0.5 * (ncol(matrix))) >= 2) {
      dds <- DESeq2::estimateSizeFactors(matrix, type = "ratio", controlGenes = CtrlGenes)
    } else {
      dds <- DESeq2::estimateSizeFactors(matrix, type = "ratio")
    }
    matrix <- DESeq2::counts(dds, normalized = TRUE)
  } else if (option == "TPM") {
    matrix <- matrix[rownames(matrix) %in% gene_length$GeneName, ]
    matrix <- matrix %>%
      DGEList() %>%
      calcNormFactors() %>%
      rpkm(gene.length = gene_length$Length[match(rownames(matrix), gene_length$GeneName)])
    matrix <- sweep(matrix, 2, apply(matrix, 2, sum), "/") * 10^6
    rownames(matrix) <- toupper(rownames(matrix))
    if (ffpe_artifacts) {
      set.seed(seed)
      ### convert to log2 scale
      matrix <- log2(matrix + 0.1)
      ### load GAM fitted model
      #model <- data("ffpemodel")
      diffmat <- matrix(
        sample(c(-1, 1), nrow(matrix) * ncol(matrix), replace = TRUE, prob = c(0.5, 0.5)) *
          rnorm(1:length(c(matrix)),
            mean = predict.gam(model, newdata = data.frame(avgtpm = c(matrix))), sd = 0.5
          ),
        nrow = nrow(matrix), ncol = ncol(matrix), byrow = FALSE
      )
      matrix <- 2^(matrix - diffmat)
    }
    ####################################################################################
    ## scRNA-seq specific
  } else if (option == "SCTransform") {
    matrix <- as(matrix, "dgCMatrix")
    matrix <- sctransform::vst(matrix, return_corrected_umi = TRUE, show_progress = FALSE)$umi_corrected
    matrix <- as(matrix, "matrix")
  } else if (option == "scran") {
    sf <- scran::computeSumFactors(as.matrix(matrix), clusters = NULL)
    sce <- SingleCellExperiment::SingleCellExperiment(assays = list(counts = as.matrix(matrix)))
    sizeFactors(sce) <- sf
    sce <- scater::normalize(sce, exprs_values = "counts", return_log = FALSE)
    matrix <- SingleCellExperiment::normcounts(sce)
  } else if (option == "scater") {
    size_factors <- scater::librarySizeFactors(matrix)
    matrix <- scater::normalizeCounts(as.matrix(matrix), size_factors = size_factors, return_log = FALSE)
  } else if (option == "Linnorm") { # It is not compatible with log transformed datasets.
    ## use Linnorm function from Linnorm package
    matrix <- expm1(Linnorm::Linnorm(as.matrix(matrix))) # Main function contains log1p(datamatrix)
  }
  return(matrix)
}

#' Deconvolution of bulk RNA-seq data
#'
#' Deconvolution of bulk RNA-seq data based on single-cell reference data. Eight bulk deconvolution methods, along with eight normalization
#' methods and four transformation methods are available.
#'
#' @param bulk a matrix or data.frame of unnormalizaed & untransformed bulk RNA-seq gene expression values with rows representing genes, columns
#' representing samples
#' @param ref a matrix or data.frame of untransformed scRNA-seq gene expression counts with rows representing genes, columns representing cells.
#' This data will be used to deconvolute provided bulk RNA-seq data.
#' @param phenodata a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first three
#' columns as:
#' \enumerate{
#'  \item cell barcodes
#'  \item cell types
#'  \item subject ids
#' }
#' @param filter_ref logical value indicating whether outlier genes & cells should be removed from the provided reference data. Defaults to TRUE
#' @param marker_genes a data.frame of two columns. First column represents cell types in ref; second column represents gene names of marker genes. If specified,
#' those genes will be used to construct signature matrix for mark-gene based deconvolution methods, such as CIBERSORT, OLS, nnls, FARDEEP and RLR. Default to NULL,
#' carry out differential analysis to identify marker genes for each cell type in ref.
#' @param genes_to_remove a vector of gene names to remove from the reference scRNA-seq data. Default to NULL.
#' @param min_pct_ct a numeric value indicating the minimum required proportion of expressing cells per cell type for marker gene identification. Only applicable when marker_genes
#' is NULL. Default to 0.05.
#' @param decon_method character value specifying the deconvolution method to use. Has to be one of "scaden", "CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR",
#' "MuSiC", "SCDC", "scTAPE". See details for more information.
#' @param norm_method_sc character value specifying the normalization method to use for reference data. Has to be one of "none","LogNormalize",
#' "SCTransform", "scran", "scater", "Linnorm". See details for more information.
#' @param norm_method_bulk character value specifying the normalization method to use for bulk data. Has to be one of "none", "TMM",
#' "median_ratios", "TPM". See details for more information.
#' @param trans_method_sc character value specifying the transformation method to use for both bulk & reference data. Has to be one of "none", "log2", "sqrt",
#' "vst". See details for more information.
#' @param trans_method_bulk character value specifying the transformation method to use for both bulk & reference data. Has to be one of "none", "log2", "sqrt",
#' "vst". See details for more information.
#' @param gene_length a data.frame with two columns. The first column represents gene names that match with provided bulk data. The second column
#' represents length of each gene. Only applicable when norm_method is selected as "TPM".
#' @param lfc_markers log2 fold change cutoff used to identify marker genes for deconvolution. The option only applicable to marker-gene based
#' approaches, such as CIBERSORT, OLS, nnls, FARDEEP and RLR. Only applicable when marker_genes is NULL.
#' @param marker_strategy further strategy in selecting marker genes besides applying the log2 fold change cutoff. Can be chosen from: "all", "pos_fc",
#' "top_50p_logFC" or "top_50p_AveExpr". See details for more information. Only applicable when marker_genes is NULL.
#' @param to_remove character value representing the cell type to remove from reference data. Only applicable to simulation experiments in evaluating
#' effect of cell type removal from reference.
#' @param ffpe_artifacts logical value indicating whether to add simulated ffpe artifacts in the bulk data. Only applicable to simulation experiments in
#' evaluating the effect of FFPE artifacts.
#' @param model pre-constructed ffpe model data. Can be downloaded from github: https://github.com/Liuy12/SCdeconR_files/blob/master/data/ffpemodel.rda.
#' @param prop a matrix or data.frame of simulated cell proportion values with rows representing cell types, columns representing samples. Only applicable to simulation
#' experiments in evaluating the effect of cell type removal from reference.
#' @param cibersortpath full path to CIBERSORT.R script.
#' @param pythonpath full path to python binary where scaden was installed with.
#' @param tmpdir temporary processing directory for scaden or scTAPE.
#' @param remove_tmpdir a logical value indicating whether to remove tmpdir once scaden is completed. Default to TRUE.
#' @param seed random seed used for simulating FFPE artifacts. Only applicable when ffpe_artifacts is set to TRUE.
#' @param nsamples number of artificial bulk samples to simulate for scaden. Default to 1000.
#' @param return_value_only return a list of values only without performing deconvolution. This could be helpful in
#' cases where the user want to apply their own deconvolution algorithms. Default to FALSE.
#' @param verbose a logical value indicating whether to print messages. Default to FALSE.
#'
#' @return a list containing two or four elements.
#' \describe{
#'  \item{first element}{a data.frame of predicted cell-type proportions, with rows representing cell types, columns representing samples.}
#'  \item{second element}{a data.frame of fitting errors of the algorithm; first column represents sample names, second column represents RMSEs.}
#'  \item{optional third element}{a data.frame of simulated cell proportion after removing the specified cell_type. Only applicable to simulation experiments.}
#'  \item{optional fourth element}{a data.frame of marker genes used for deconvolution. Only applicable to marker-gene based deconvolution methods.}
#' }
#'
#' @details
#' decon_method should be one of the following:
#' \describe{
#'  \item{\href{https://github.com/KevinMenden/scaden}{scaden}}{a deep learning based method using three multi-layer deep neural nets. To use scaden,
#' you need to firstly install scaden via \code{pip} or \code{conda}, the provide the python path to \code{pythonpath} option.}
#'  \item{CIBERSORT}{a marker gene based support vectors regression approach. CIBERSOR does not allow redistribution. To use CIBERSORT,
#' you need to request the source code from the authors & provide the path of CIBERSORT.R script to \code{cibersortpath} option.}
#'  \item{OLS}{ordinary least squares.}
#'  \item{\href{https://CRAN.R-project.org/package=nnls}{nnls}}{non-negative least squares.}
#'  \item{\href{https://CRAN.R-project.org/package=FARDEEP}{FARDEEP}}{robust regression using least trimmed squares}
#'  \item{\href{https://CRAN.R-project.org/package=MASS}{RLR}}{robust regression using an M estimator}
#'  \item{\href{https://github.com/xuranw/MuSiC}{MuSiC}}{multi-subject single-cell deconvolution}
#'  \item{\href{https://github.com/meichendong/SCDC}{SCDC}}{an ENSEMBLE method to integrate deconvolution results from different scRNA-seq datasets}
#'  \item{\href{https://github.com/poseidonchan/TAPE}{scTAPE}}{Deep autoencoder based deconvolution}
#' }
#'
#' norm_method should be one of the following:
#' \describe{
#'  \item{none}{no normalization is performed.}
#'  \item{LogNormalize}{\code{\link[Seurat]{LogNormalize}} method from seurat.}
#'  \item{TMM}{TMM method from \code{\link[edgeR]{calcNormFactors}} function from edgeR.}
#'  \item{median_ratios}{median ratio method from \code{\link[DESeq2]{estimateSizeFactors,DESeqDataSet-method}} function from DESeq2.}
#'  \item{TPM}{Transcript per million. TPM has to be chosen if ffpe_artifacts is set to TRUE.}
#'  \item{SCTransform}{\code{\link[Seurat]{SCTransform}} method from Seurat.}
#'  \item{scran}{\code{\link[scran]{computeSumFactors}} method from scran.}
#'  \item{scater}{\code{\link[scater]{librarySizeFactors}} method from scater.}
#'  \item{Linnorm}{\code{Linnorm} method from Linnorm.}
#' }
#'
#' trans_method should be one of the following:
#' \describe{
#'  \item{none}{no transformation is performed.}
#'  \item{log2}{log2 transformation. 0.1 is added to the data to avoid logarithm of 0s.}
#'  \item{sqrt}{square root transformation.}
#'  \item{vst}{\code{\link[DESeq2]{varianceStabilizingTransformation}} method from DESeq2.}
#' }
#'
#' marker_strategy should be one of the following:
#' \describe{
#'  \item{all}{all genes passed fold change threshold will be used.}
#'  \item{pos_fc}{only genes with positive fold changes will be used.}
#'  \item{top_50p_logFC}{only genes with top 50 percent positive fold changes will be used.}
#'  \item{top_50p_AveExpr}{only genes with top 50 percent average expression will be used.}
#' }
#' @export
#'
#' @examples
#' \dontrun{
#' ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
#'               paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#'
#' # construct integrated reference using harmony algorithm
#' refdata <- construct_ref(ref_list = ref_list,
#'                       phenodata_list = phenodata_list,
#'                       data_type = "cellranger",
#'                       method = "harmony",
#'                       group_var = "subjectid",
#'                       nfeature_rna = 50,
#'                       vars_to_regress = "percent_mt", verbose = FALSE)
#'
#' phenodata <- data.frame(cellid = colnames(refdata),
#'                         celltypes = refdata$celltype,
#'                         subjectid = refdata$subjectid)
#' bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                            phenodata = phenodata,
#'                            num_mixtures = 20,
#'                            num_mixtures_sprop = 1)
#'
#' ## perform deconvolution based on "OLS" algorithm
#' decon_res <- scdecon(bulk = bulk_sim[[1]],
#'                      ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                      phenodata = phenodata,
#'                      filter_ref = TRUE,
#'                      decon_method = "OLS",
#'                      norm_method_sc = "LogNormalize",
#'                      norm_method_bulk = "TMM",
#'                      trans_method_sc = "none",
#'                      trans_method_bulk = "log2",
#'                      marker_strategy = "all")
#' }


scdecon <- function(
    bulk,
    ref,
    phenodata,
    filter_ref = TRUE,
    marker_genes = NULL,
    genes_to_remove = NULL,
    min_pct_ct = 0.05,
    decon_method = c("scaden", "CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR", "MuSiC", "SCDC", "scTAPE"),
    norm_method_sc = c("none","LogNormalize", "SCTransform", "scran", "scater", "Linnorm"),
    norm_method_bulk = c("none", "TMM", "median_ratios", "TPM"),
    trans_method_sc = c("none", "log2", "sqrt", "vst"),
    trans_method_bulk = c("none", "log2", "sqrt", "vst"),
    gene_length = NULL,
    lfc_markers = log2(1.5),
    marker_strategy = c("all", "pos_fc", "top_50p_logFC", "top_50p_AveExpr"),
    to_remove = NULL,
    ffpe_artifacts = FALSE,
    model = NULL,
    prop = NULL,
    cibersortpath = NULL,
    pythonpath = NULL,
    tmpdir = NULL,
    remove_tmpdir = TRUE,
    seed = 1234,
    nsamples = 1000,
    return_value_only = FALSE,
    verbose = FALSE) {
  decon_method_all <- c("CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR", "MuSiC", "SCDC", "scaden", "scTAPE")
  decon_method_sc_all <- c("MuSiC", "SCDC", "scaden", "scTAPE")
  decon_method_bulk_all <- c("CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR")
  norm_method_sc_all <- c("none","LogNormalize", "SCTransform", "scran", "scater", "Linnorm")
  norm_method_bulk_all <- c("none", "TMM", "median_ratios", "TPM")
  trans_method_all <- c("none", "log2", "sqrt", "vst")
  marker_strategy_all <- c("all", "pos_fc", "top_50p_logFC", "top_50p_AveExpr")
  if (!decon_method %in% decon_method_all) stop(paste0("decon_method must be one of ", decon_method_all, collapse = ","))
  if (!norm_method_sc %in% norm_method_sc_all) stop(paste0("norm_method_sc must be one of ", paste0(norm_method_sc_all, collapse = ",")))
  if (!norm_method_bulk %in% norm_method_bulk_all) stop(paste0("norm_method_bulk must be one of ", paste0(norm_method_bulk_all, collapse = ",")))
  if (!trans_method_sc %in% trans_method_all) stop(paste0("trans_method_sc must be one of ", paste0(trans_method_all, collapse = ",")))
  if (!trans_method_bulk %in% trans_method_all) stop(paste0("trans_method_bulk must be one of ", paste0(trans_method_all, collapse = ",")))
  if (decon_method %in% decon_method_bulk_all && (!marker_strategy %in% marker_strategy_all)) stop(paste0("marker_strategy must be one of ", paste0(marker_strategy_all, collapse = ",")))
  if (decon_method == "scaden" | decon_method == "scTAPE"){
    if (is.null(pythonpath) && reticulate::py_available()) {
      warning(paste0("pythonpath not specified, will use the python found here: ", reticulate::py_config()$python))
      pythonpath <- reticulate::py_config()$python
    } else if(is.null(pythonpath) && (!reticulate::py_available())) stop("pythonpath not specified, and python cannot be found via py_available()")
    reticulate::use_python(pythonpath)
    if(decon_method == "scaden" && (!reticulate::py_module_available("scaden"))) stop("scaden not installed. You can install scaden via pip or conda. See installation page for details")
    if(decon_method == "scTAPE" && (!reticulate::py_module_available("TAPE"))) stop("scTAPE not installed. You can install scTAPE via pip or conda. See installation page for details")
  }
  if( decon_method == "CIBERSORT" && (is.null(cibersortpath) || (!file.exists(cibersortpath)))) stop("cibersortpath not provided.")
  if (ncol(phenodata) < 3) stop("phenodata should contain at least the first three columns: cellid, celltype and subjectid.")
  colnames(phenodata)[1:3] <- c("cellid", "celltype", "subjectid")
  if (length(unique(phenodata$cellid)) != nrow(phenodata)) stop("values of cellid in phenodata not unique.")
  if (length(intersect(colnames(ref), phenodata$cellid)) != length(union(colnames(ref), phenodata$cellid))) stop("column names of reference data do not match with cellid of the phenodata.")
  if ((!is.null(to_remove)) && (!to_remove %in% phenodata$celltype)) stop("to_remove not present in celltype of phenodata")
  if (decon_method %in% decon_method_bulk_all) decon_type <- "bulk" else decon_type <- "sc"
  if (norm_method_bulk == "TPM") {
    if(is.null(gene_length)) {
      stop("norm_method is specified as TPM, but no gene_length is provided")
    } else if(ncol(gene_length) != 2) {
      stop("gene_length need to have two columns, name of genes & length of genes")
    } else if(length(intersect(rownames(bulk), gene_length[[1]])) != length(union(rownames(bulk), gene_length[[1]]))){
      stop("gene names for gene_length are not consistent with row names of bulk")
    }
    colnames(gene_length) <- c('GeneName', 'Length')
  }
  if(!length(intersect(rownames(ref), rownames(bulk)))) stop("no overlapping genes/rownames between ref and bulk")
  if((!is.null(marker_genes)) && (ncol(marker_genes) !=2)) stop("marker_genes need to be a data.frame with two columns (cell type, gene name)")
  if((!is.null(marker_genes)) && (!length(Reduce(intersect, list(marker_genes[,2], rownames(bulk), rownames(ref)))))) stop("no overlapping genes between supplied marker gene list and ref/bulk")
  phenodata <- phenodata[match(colnames(ref), phenodata$cellid), ]
  rownames(phenodata) <- phenodata$cellid
  if (filter_ref) {
    if(verbose) message("filtering reference data.")
    lib_sizes <- colSums(ref)
    gene_names <- rownames(ref)
    mt_id <- grepl("^MT-|_MT-", gene_names, ignore.case = TRUE)
    cellstoremove <- c()
    if(length(which(mt_id))){
      mt_percent <- colSums(ref[mt_id, ]) / lib_sizes
      cellstoremove <- filter_cells(mt_percent)
    }
    rb_id <- grepl("^RPL|^RPS|_RPL|_RPS", gene_names, ignore.case = TRUE)
    if(length(which(rb_id))){
    rb_percent <- colSums(ref[rb_id, ]) / lib_sizes
    cellstoremove <- c(cellstoremove, filter_cells(rb_percent))
    }
    cellstoremove <- unique(c(cellstoremove,  filter_cells(lib_sizes)))
    if (length(cellstoremove) != 0) {
      ref <- ref[, -cellstoremove]
      phenodata <- phenodata[-cellstoremove, ]
    }
    keep <- sapply(unique(phenodata$celltype), function(i) {
      ct_hits <- which(phenodata$celltype == i)
      size <- ceiling(min_pct_ct * length(ct_hits))
      rowSums(ref[, ct_hits, drop = FALSE] != 0) >= size
    })
    ref <- ref[rowSums(keep) > 0, ]
  }
  if(!is.null(genes_to_remove)) ref <- ref[!rownames(ref) %in% genes_to_remove,]
  if(decon_type == "bulk"){
    celltypes <- phenodata$celltype
    avgexp_ct <- lapply(unique(celltypes), function(i) rowMeans(ref[, celltypes == i]))
    avgexp_ct <- do.call(cbind.data.frame, avgexp_ct)
    colnames(avgexp_ct) <- unique(celltypes)
    if(is.null(marker_genes)){
      if(verbose) message("find cell type marker genes using limma")
      keep <- sapply(unique(celltypes), function(i) {
        ct_hits <- which(celltypes == i)
        size <- ceiling(min_pct_ct * length(ct_hits))
        rowSums(ref[, ct_hits, drop = FALSE] != 0) >= size
      })
      ref_sel <- ref[rowSums(keep) > 0, ]
      ref_sel <- Normalization(ref_sel)
      annotation <- factor(celltypes)
      design <- model.matrix(~ 0 + annotation)
      colnames(design) <- gsub("annotation", "", colnames(design))
      cont_matrix <- matrix((-1 / ncol(design)), nrow = ncol(design), ncol = ncol(design))
      colnames(cont_matrix) <- colnames(design)
      diag(cont_matrix) <- (ncol(design) - 1) / ncol(design)
      tmp <- limma::voom(ref_sel, design = design, plot = FALSE)
      fit <- limma::lmFit(tmp, design)
      fit2 <- limma::contrasts.fit(fit, cont_matrix)
      fit2 <- limma::eBayes(fit2, trend = TRUE)
      markers <- markerfc(fit2, log2_threshold = lfc_markers)
      if(length(unique(markers$CT)) != length(unique(phenodata$celltype))) warning(paste0("some cell types don't have marker genes: ", paste0(setdiff(unique(phenodata$celltype), unique(markers$CT)), collapse = ',')))
    } else {
      colnames(marker_genes) <- c("CT", "gene")
      if(length(unique(marker_genes$CT)) != length(unique(phenodata$celltype))) warning(paste0("some cell types don't have marker genes: ", paste0(setdiff(unique(phenodata$celltype), unique(marker_genes$CT)), collapse = ',')))
      }
    }
  if (decon_type == "bulk") {
    if(verbose) message(paste0("perform transformation and normalization."))
    bulk <- scaling(bulk, norm_method_bulk, ffpe_artifacts = ffpe_artifacts, gene_length = gene_length, model = model)
    avgexp_ct <- scaling(avgexp_ct, norm_method_sc, ffpe_artifacts = FALSE, gene_length = gene_length)
    bulk <- transformation(bulk, trans_method_bulk)
    avgexp_ct <- transformation(avgexp_ct, trans_method_sc)
    if(is.null(marker_genes)) marker_distrib <- marker_strategies(markers, marker_strategy) else marker_distrib <- marker_genes
    if(!is.null(to_remove)){
      if(verbose) message(paste0("remove specified cell type: ", to_remove, "."))
      bulk <- bulk[,prop[to_remove,] != 0]
      avgexp_ct <- avgexp_ct[, colnames(avgexp_ct) %in% rownames(prop) & (!colnames(avgexp_ct) %in% to_remove)]
      prop <- prop[!rownames(prop) %in% to_remove, colnames(bulk)]
      marker_distrib <- marker_distrib[marker_distrib$CT %in% rownames(prop) & (marker_distrib$CT != to_remove),]
    }
    if(return_value_only){
      message("return values without performing deconvlution")
      return(list(bulk = bulk, ref = avgexp_ct, marker_distrib = marker_distrib))
    }
    if(verbose) message(paste0("perform deconvolution analysis using ", decon_method, "."))
    results <- deconvolution(bulk = bulk, ref = avgexp_ct, decon_method = decon_method, marker_distrib = marker_distrib, cibersortpath = cibersortpath, verbose = verbose)
  } else if (decon_type == "sc") {
    if(verbose) message(paste0("perform transformation and normalization"))
    bulk <- scaling(bulk, norm_method_bulk, ffpe_artifacts = ffpe_artifacts, gene_length = gene_length, model = model)
    ref <- scaling(ref, norm_method_sc, ffpe_artifacts = FALSE, gene_length = gene_length)
    bulk <- transformation(bulk, trans_method_bulk)
    ref <- transformation(ref, trans_method_sc)
    if(!is.null(to_remove)){
      if(verbose) message(paste0("remove specified cell type: ", to_remove, "."))
      bulk <- bulk[,prop[to_remove,] != 0]
      ref <- ref[,phenodata$celltype != to_remove]
      prop <- prop[!rownames(prop) %in% to_remove, colnames(bulk)]
      phenodata <- phenodata[phenodata$celltype != to_remove,]
    }
    if(return_value_only){
      message("return values without performing deconvlution")
      return(list(bulk = bulk, ref = ref, phenodata = phenodata))
    }
    if(verbose) message(paste0("perform deconvolution analysis using ", decon_method, "."))
    results <- deconvolution(bulk = bulk, ref = ref, decon_method = decon_method, phenodata = phenodata, pythonpath = pythonpath, tmpdir = tmpdir, remove_tmpdir = remove_tmpdir, verbose = verbose, nsamples = nsamples)
  }
  output <- vector(mode = "list", length = 4)
  output[[1]] <- results[[1]]
  output[[2]] <- results[[2]]
  if(!is.null(prop)) output[[3]] <- prop
  if(decon_type == "bulk") output[[4]] <- marker_distrib
  names(output) <- c("prediction", "fit_error", "proportion", "markers")
  return(output)
}


#' Bar plot of cell type proportions across samples
#'
#' Bar plot of cell type proportions across samples
#'
#' @param prop a matrix or data.frame of cell proportion values with rows representing cell types, columns representing samples.
#' @param sort a logical value indicating whether to sort the samples based on cell type with highest median cell proportion across samples. Default to TRUE.
#' @param interactive a logical value indicating whether to generate interactive plot. Default to FALSE.
#' @export
#'

prop_barplot <- function(prop, sort = TRUE, interactive = FALSE){
  prop <- as.data.frame(t(prop))
  prop <-  prop %>% mutate(sampleid = rownames(prop)) %>%
  reshape2::melt(id.var = "sampleid", value.name = "ct_prop", variable.name = "ct") %>% as.data.frame()
 if(sort) {
  prop_median <- prop %>% group_by(ct) %>% summarise(median_prop = median(ct_prop))
  ct_sel <- prop_median$ct[which.max(prop_median$median_prop)]
  prop$sampleid <- factor(prop$sampleid, levels = prop$sampleid[prop$ct == ct_sel][order(prop$ct_prop[prop$ct == ct_sel],decreasing = T)])
  prop$ct <- factor(prop$ct, levels = prop_median$ct[order(prop_median$median_prop, decreasing = FALSE)])
  }
  gp <- ggplot() + geom_col(aes(x = sampleid,y = ct_prop, fill = ct), data = prop) +
  theme_classic() + labs(y='Predicted proportion', fill = "Cell types") +
  theme(axis.title = element_text(size=20,face='bold'),axis.text = element_text(size=15),axis.text.x = element_blank(),
        axis.ticks.x=element_blank(),legend.title = element_text(size=15,face='bold'),legend.text = element_text(size=12))
  if(interactive) return(plotly::ggplotly(gp)) else return(gp)
}


deconvolution <- function(bulk, ref, decon_method, phenodata, marker_distrib, pythonpath = NULL, cibersortpath = NULL, tmpdir = NULL, remove_tmpdir = TRUE, verbose = FALSE, nsamples = 1000) {
  bulk_methods <- c("CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR")
  sc_methods <- c("MuSiC", "SCDC", "scaden", "scTAPE")

  ########## Using marker information for bulk_methods
  if (decon_method %in% bulk_methods) {
    ref <- ref[rownames(ref) %in% marker_distrib$gene, ]
    bulk <- bulk[rownames(bulk) %in% marker_distrib$gene, ]
  } else if (decon_method %in% sc_methods) {
    if (ncol(phenodata) < 3) stop("phenodata should contain at least the first three columns: cellid, celltype and subjectid.")
    colnames(phenodata)[1:3] <- c("cellid", "celltype", "subjectid")
    if (length(unique(phenodata$cellid)) != nrow(phenodata)) stop("values of cellid in phenodata not unique.")
    if (length(intersect(colnames(ref), phenodata$cellid)) != length(union(colnames(ref), phenodata$cellid))) stop("column names of reference data do not match with cellid of the phenodata.")
    phenodata <- phenodata[match(colnames(ref), phenodata$cellid), ]
    rownames(phenodata) <- phenodata$cellid
  }
  keep <- intersect(rownames(ref), rownames(bulk))
  if(verbose) message(length(keep), " number of genes overlapping between ref and bulk data.")
  ref <- ref[keep, ]
  bulk <- bulk[keep, ]
  if(decon_method %in% c("SCDC", "MuSiC")) {
    if(verbose) message("construct eset object.")
    ref_eset <- Biobase::ExpressionSet(assayData = as.matrix(ref), phenoData = Biobase::AnnotatedDataFrame(phenodata))
    bulk_eset <- Biobase::ExpressionSet(assayData = as.matrix(bulk))
  }
  ###################################
  if (decon_method == "CIBERSORT") { # without QN. By default, CIBERSORT performed QN (only) on the mixture.
    source(cibersortpath)
    #results <- CIBERSORT(sig_matrix = ref, mixture_file = bulk, QN = FALSE)
    ## to get away with check note
    results <- eval(parse(text = paste0("CIBERSORT(sig_matrix = ref, mixture_file = bulk, QN = FALSE)")))
    fiterror <- data.frame(sample = rownames(results), RMSE = results[, ncol(results)])
    results <- t(results[, 1:(ncol(results) - 3)])
  } else if (decon_method == "OLS") {
    results <- apply(bulk, 2, function(x) lm(x ~ as.matrix(ref))$coefficients[-1])
    results <- apply(results, 2, function(x) ifelse(x < 0, 0, x)) # explicit non-negativity constraint
    results <- apply(results, 2, function(x) x / sum(x)) # explicit STO constraint
    fiterror <- data.frame(
      sample = colnames(results),
      RMSE = sapply(1:ncol(bulk), function(i) {
        u <- sweep(ref, MARGIN = 2, results[, i], "*")
        k <- apply(u, 1, sum)
        sqrt((mean((k - bulk[, i])^2)))
      })
    )
    rownames(results) <- unlist(lapply(strsplit(rownames(results), ")"), function(x) x[2]))
  } else if (decon_method == "nnls") {
    results <- do.call(cbind.data.frame, lapply(apply(bulk, 2, function(x) nnls::nnls(as.matrix(ref), x)), function(y) y$x))
    results <- apply(results, 2, function(x) x / sum(x)) # explicit STO constraint
    rownames(results) <- colnames(ref)
    fiterror <- data.frame(sample = colnames(results), RMSE = sapply(1:ncol(bulk), function(i) {
      u <- sweep(ref, MARGIN = 2, results[, i], "*")
      k <- apply(u, 1, sum)
      sqrt((mean((k - bulk[, i])^2)))
    }))
  } else if (decon_method == "FARDEEP") {
    ## call fardeep from FARDEEP
    results <- t(FARDEEP::fardeep(ref, bulk, nn = TRUE, intercept = TRUE, permn = 10, QN = FALSE)$abs.beta)
    results <- apply(results, 2, function(x) x / sum(x)) # explicit STO constraint
    fiterror <- data.frame(sample = colnames(results), RMSE = sapply(1:ncol(bulk), function(i) {
      u <- sweep(ref, MARGIN = 2, results[, i], "*")
      k <- apply(u, 1, sum)
      sqrt((mean((k - bulk[, i])^2)))
    }))
  } else if (decon_method == "RLR") { # RLR = robust linear regression
    results <- do.call(cbind.data.frame, lapply(apply(bulk, 2, function(x) MASS::rlm(x ~ as.matrix(ref), maxit = 100)), function(y) y$coefficients[-1]))
    results <- apply(results, 2, function(x) ifelse(x < 0, 0, x)) # explicit non-negativity constraint
    results <- apply(results, 2, function(x) x / sum(x)) # explicit STO constraint
    rownames(results) <- unlist(lapply(strsplit(rownames(results), ")"), function(x) x[2]))
    fiterror <- data.frame(sample = colnames(results), RMSE = sapply(1:ncol(bulk), function(i) {
      u <- sweep(ref, MARGIN = 2, results[, i], "*")
      k <- apply(u, 1, sum)
      sqrt((mean((k - bulk[, i])^2)))
    }))
  } else if (decon_method == "MuSiC") {
    #results <- t(MuSiC::music_prop(
    #  bulk_eset, ref_eset, clusters = "celltype",
    #  markers = NULL, normalize = FALSE, samples = "subjectid",
    #  verbose = FALSE
    #)$Est.prop.weighted)
    results <- eval(parse(text = 't(MuSiC::music_prop(bulk_eset, ref_eset, clusters = "celltype", markers = NULL, normalize = FALSE, samples = "subjectid", verbose = FALSE)$Est.prop.weighted)'))
    fiterror <- data.frame(sample = colnames(results), RMSE = sapply(1:ncol(bulk), function(i) {
      u <- sweep(ref, MARGIN = 2, results[, i], "*")
      k <- apply(u, 1, sum)
      sqrt((mean((k - bulk[, i])^2)))
    }))
  } else if (decon_method == "SCDC") {
    ### use SCDC_prop from SCDC package
    #results <- t(SCDC::SCDC_prop(bulk.eset = bulk_eset, sc.eset = ref_eset, ct.varname = "celltype", sample = "subjectid", ct.sub = unique(as.character(phenodata$celltype)), iter.max = 200)$prop.est.mvw)
    ### to get away with check note
    results <- eval(parse(text = 't(SCDC::SCDC_prop(bulk.eset = bulk_eset, sc.eset = ref_eset, ct.varname = "celltype", sample = "subjectid", ct.sub = unique(as.character(phenodata$celltype)), iter.max = 200)$prop.est.mvw)'))
    fiterror <- data.frame(sample = colnames(results), RMSE = sapply(1:ncol(bulk), function(i) {
      u <- sweep(ref, MARGIN = 2, results[, i], "*")
      k <- apply(u, 1, sum)
      sqrt((mean((k - bulk[, i])^2)))
    }))
  } else if (decon_method == "scaden") {
    if(is.null(tmpdir)) {
      warning("tmpdir not supplied. Creating tmpdir in current working directory.")
      tmpdir <- paste0(getwd(), "/tmpdir/")
      dir.create(tmpdir)
    } else if(!dir.exists(tmpdir)) stop("tmpdir does not exist")
    else if(length(list.files(tmpdir)) > 0) stop("tmpdir exists, but is not empty.")
    cwd <- getwd()
    on.exit(setwd(cwd))
    setwd(tmpdir)
    data.table::fwrite(as.matrix(Matrix::t(ref)), "./decon_counts.txt", col.names = TRUE, row.names = TRUE, sep = "\t", quote = FALSE)
    data.table::fwrite(data.frame(Celltype = phenodata$celltype), "./decon_celltypes.txt", col.names = TRUE, row.names = FALSE, sep = "\t", quote = FALSE)
    data.table::fwrite(as.data.frame(bulk), "./decon_bulk_data.txt", col.names = TRUE, row.names = TRUE, sep = "\t", quote = FALSE)
    if(verbose) message("running scaden, and it might take a while. Set verbose = TRUE to track progress. ")
    system(paste0("scaden simulate --data ./ --pattern '*_counts.txt'", " -n ", nsamples), ignore.stdout = !verbose, ignore.stderr = !verbose)
    system(paste0("scaden process data.h5ad decon_bulk_data.txt"), ignore.stdout = !verbose, ignore.stderr = !verbose)
    system(paste0("scaden train processed.h5ad --steps 5000 --model_dir model"), ignore.stdout = !verbose, ignore.stderr = !verbose)
    system(paste0("scaden predict --model_dir model decon_bulk_data.txt"), ignore.stdout = !verbose, ignore.stderr = !verbose)
    results <- t(read.delim("./scaden_predictions.txt", header = TRUE, row.names = 1))
    fiterror <- NA
    if(remove_tmpdir) system(paste0("rm -rf ", tmpdir))
  } else if(decon_method == "scTAPE"){
    if(is.null(tmpdir)) {
      warning("tmpdir not supplied. Creating tmpdir in current working directory.")
      tmpdir <- paste0(getwd(), "/tmpdir/")
      dir.create(tmpdir)
    } else if(!dir.exists(tmpdir)) stop("tmpdir does not exist")
    else if(length(list.files(tmpdir)) > 0) stop("tmpdir exists, but is not empty.")
    cwd <- getwd()
    on.exit(setwd(cwd))
    setwd(tmpdir)
    sc_data <- t(ref)
    sc_data <- cbind(data.frame(Celltype = phenodata$celltype), sc_data)
    data.table::fwrite(sc_data, paste0(tmpdir, "/sc_data.txt"), sep = "\t", row.names = FALSE, col.names = TRUE)
    write.table(t(bulk),paste0(tmpdir,"/bulk_data.txt"), sep = "\t", row.names = TRUE, col.names = NA, quote = F)
    tape_script <- paste0(find.package("SCdeconR"), "/script/tape_deconvolution.py")
    #system(paste0("cp ", tape_script, " ", tmpdir))
    reticulate::source_python(tape_script)
    # get away with check note
    #results <- t(tape_deconvolution(sc_data = paste0(tmpdir, "/sc_data.txt"), bulk_data = paste0(tmpdir,"/bulk_data.txt")))
    results <- eval(parse(text = paste0('t(tape_deconvolution(sc_data = ', tmpdir, '/sc_data.txt, bulk_data = ', tmpdir,'/bulk_data.txt')))
    fiterror <- NA
    if(remove_tmpdir) system(paste0("rm -rf ", tmpdir))
  }
  return(list(results, fiterror))
}

utils::globalVariables(c("log2foldchange.x",
                         "log2foldchange.y",
                         "category",
                         "genename",
                         "NES.x",
                         "NES.y",
                         "NAME",
                         "celltype",
                         "mixture_samep",
                         "observed_values",
                         "expected_values",
                         "mixture",
                         "i",
                         "lfcSE",
                         "padj",
                         "pvalue",
                         "baseMean",
                         "log2foldchange",
                         "log2FoldChange",
                         "pval",
                         "logCPM",
                         "PValue",
                         "pval",
                         "gsearespath",
                         "NES",
                         "SIZE",
                         "adj.P.Val",
                         "P.Value",
                         "AveExpr",
                         "padj_method",
                         "log2FC",
                         "CT",
                         "ct",
                         "ct_prop",
                         "sampleid"
                         ))

#' Generate artificial bulk RNA-seq samples based on simulation
#'
#' Generate artificial bulk RNA-seq samples with random or pre-defined cell-type proportions for benchmarking deconvolution algorithms
#'
#' @param ref a matrix-like object of gene expression values with rows representing genes, columns representing cells.
#' @param phenodata a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first two columns as:
#' \enumerate{
#'  \item cell barcodes
#'  \item cell types
#' }
#' @param num_mixtures total number of simulated bulk samples. Have to be multiple of \code{num_mixtures_sprop}. Default to 500.
#' @param num_mixtures_sprop number of simulated bulk samples with the same simulated cell type proportions. Only applicable when \code{prop} is not specified.
#' Those samples will be used to estimate bias & variance. Default to 10.
#' @param pool_size number of cells to use to construct each artificial bulk sample. Default to 100.
#' @param seed seed to use for simulation. Default to 1234.
#' @param prop a data.frame with two columns. The first column includes unique cell types in phenodata; the second column includes cell type proportions.
#' If specified, bulk samples will be simulated based on the specified cell proportions.
#' @param replace logical value indicating whether to sample cells with replacement. Default to FALSE, to sample cells without replacement.
#'
#' @return a list of two objects:
#' \enumerate{
#'  \item simulated bulk RNA-seq data, with rows representing genes, columns representing samples
#'  \item cell type proportions used to simulate the bulk RNA-seq data, with rows representing cell types, columns representing samples
#' }
#'
#' @details
#' If \code{prop} is not specified, cell type proportions will be firstly randomly generated with at least two cell types present. Then, for each cell proportion
#' vector, \code{num_mixtures_sprop} number of samples is simulated. Eventually, a total of \code{num_mixtures} number of samples is simulated. If prop is
#' specified, then a total of \code{num_mixtures} number of samples will be simulated based on the same cell proportion vector specified.
#'
#' @export
#'
#' @examples
#' \donttest{
#' ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
#'               paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#'
#' # construct integrated reference using harmony algorithm
#' refdata <- construct_ref(ref_list = ref_list,
#'                       phenodata_list = phenodata_list,
#'                       data_type = "cellranger",
#'                       method = "harmony",
#'                       group_var = "subjectid",
#'                       nfeature_rna = 50,
#'                       vars_to_regress = "percent_mt", verbose = FALSE)
#' phenodata <- data.frame(cellid = colnames(refdata),
#'                         celltypes = refdata$celltype,
#'                         subjectid = refdata$subjectid)
#' prop <- data.frame(celltypes = unique(refdata$celltype), 
#' proportion = rep(1/length(unique(refdata$celltype)), length(unique(refdata$celltype))))
#' bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                            phenodata = phenodata,
#'                            num_mixtures = 20,
#'                            prop = prop,
#'                            num_mixtures_sprop = 1)
#' }


bulk_generator <- function(
    ref,
    phenodata,
    num_mixtures = 500,
    num_mixtures_sprop = 10,
    pool_size = 100,
    seed = 1234,
    prop = NULL,
    replace = FALSE){
  colnames(phenodata)[1:2] <- c("cellid", "celltype")
  if (length(unique(phenodata$cellid)) != nrow(phenodata)) stop("values of cellid in phenodata not unique.")
  if (length(intersect(colnames(ref), phenodata$cellid)) != length(union(colnames(ref), phenodata$cellid))) stop("column names of reference data do not match with cellid of the phenodata.")
  if (length(unique(phenodata$celltype)) < 2) stop("At least two unique cell types need to present in phenodata.")
  set.seed(seed)
  celltypes <- unique(phenodata$celltype)

  if (!is.null(prop)) {
    colnames(prop) <- c("ct", "expected")
    bulk <- lapply(1:num_mixtures, function(i) {
      chosen_cells <- sapply(which(prop$expected != 0), function(x) {
        ncells <- prop$expected[x] * pool_size
        chosen <- sample(
          phenodata$cellid[phenodata$celltype == prop$ct[x]],
          ncells, replace = replace
        )
        chosen
      }) %>% unlist()
      rowSums(ref[, colnames(ref) %in% chosen_cells]) %>% as.data.frame()
    })
    bulk <- do.call(cbind, bulk)
    colnames(bulk) <- paste("mix1_", 1:num_mixtures, sep = "")
    ## set CT as rownames
    rownames(prop) <- prop$ct
    prop <- prop[, rep(2, num_mixtures)]
    colnames(prop) <- colnames(bulk)
  } else {
    cell_distribution <- data.frame(table(phenodata$celltype), stringsAsFactors = FALSE)
    colnames(cell_distribution) <- c("ct", "maxn")

    tissues <- list()
    proportions <- list()
    rm(list = "prop")

    for (y in 1:(num_mixtures / num_mixtures_sprop)) {
      # Only allow feasible mixtures based on cell distribution
      while (!exists("prop")) {
        num_ct_mixture <- sample(x = 2:length(celltypes), 1)
        selectedct <- sample(celltypes, num_ct_mixture, replace = FALSE)

        prop <- runif(num_ct_mixture, 1, 99)
        prop <- round(prop / sum(prop), digits = log10(pool_size)) # sum to 1
        prop <- data.frame(ct = selectedct, expected = prop, stringsAsFactors = FALSE)

        missingct <- celltypes[!celltypes %in% selectedct]
        missingct <- data.frame(ct = missingct, expected = rep(0, length(missingct)), stringsAsFactors = FALSE)

        prop <- rbind.data.frame(prop, missingct)
        potential_mix <- merge(prop, cell_distribution)
        potential_mix$size <- potential_mix$expected * pool_size

        if (!all(potential_mix$maxn >= potential_mix$size) || sum(prop$expected) != 1) {
          rm(list = "prop")
        }
      }
      # Using info in prop to build bulk simultaneously
      # randomly sample 10 times to estimate bias and variance for each vector of prop
      bulk <- lapply(1:num_mixtures_sprop, function(i) {
        chosen_cells <- sapply(which(prop$expected != 0), function(x) {
          ncells <- prop$expected[x] * pool_size
          chosen <- sample(
            phenodata$cellid[phenodata$celltype == prop$ct[x]],
            ncells, replace = replace
          )
          chosen
        }) %>% unlist()
        rowSums(ref[, colnames(ref) %in% chosen_cells]) %>% as.data.frame()
      })
      bulk <- do.call(cbind, bulk)
      colnames(bulk) <- paste("mix", y, "_", 1:num_mixtures_sprop, sep = "")
      prop <- prop[, c("ct", "expected")]
      propn <- nrow(prop)
      prop <- prop[rep(seq_len(propn), times = num_mixtures_sprop), ]
      prop$mix <- paste("mix", y, "_", rep(1:num_mixtures_sprop, each = propn), sep = "")
      tissues[[y]] <- bulk
      proportions[[y]] <- prop

      rm(list = c("bulk", "prop", "missingct"))
    }

    prop <- do.call(rbind.data.frame, proportions)
    bulk <- do.call(cbind.data.frame, tissues)

    prop <- reshape2::dcast(prop, ct ~ mix,
      value.var = "expected",
      fun.aggregate = sum
    )
    prop <- as.data.frame(prop, row.names = 1)
    prop <- prop[, mixedsort(colnames(prop))]
  }
  return(list(bulk = bulk, prop = prop))
}

#' Statistical evaluations of predicted cell proportions
#'
#' Compute RMSE, bias & variance metrics for predicted cell proportions by comparing with expected cell proportions.
#'
#' @param prop_pred a matrix-like object of predicted cell proportion values with rows representing cell types, columns representing samples.
#' @param prop_sim a matrix-like object of simulated/expected cell proportion values with rows representing cell types, columns representing samples.
#'
#' @return a list of two objects:
#' \enumerate{
#'  \item a data.fame of summary metrics containing RMSE, bias & variance grouped by cell types and mixture ids (simulated samples with the same expected cell proportions).
#'  \item a data.frame of aggregated RMSE values across all cell types within each sample.
#' }
#'
#' @export
#'
#' @examples
#' \donttest{
#' ## generate artificial bulk samples
#' ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
#'               paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
#' phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample1.txt")
#' phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
#' "/refdata/phenodata_sample2.txt")
#' phenodata_list <- c(phenopath1,phenopath2)
#'
#' # construct integrated reference using harmony algorithm
#' refdata <- construct_ref(ref_list = ref_list,
#'                       phenodata_list = phenodata_list,
#'                       data_type = "cellranger",
#'                       method = "harmony",
#'                       group_var = "subjectid",
#'                       nfeature_rna = 50,
#'                       vars_to_regress = "percent_mt", verbose = FALSE)
#' phenodata <- data.frame(cellid = colnames(refdata),
#'                         celltypes = refdata$celltype,
#'                         subjectid = refdata$subjectid)
#' prop <- data.frame(celltypes = unique(refdata$celltype), 
#' proportion = rep(1/length(unique(refdata$celltype)), length(unique(refdata$celltype))))   
#' bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                            phenodata = phenodata,
#'                            num_mixtures = 20,
#'                            prop = prop,
#'                            num_mixtures_sprop = 1)
#'
#' ## perform deconvolution based on "OLS" algorithm
#' decon_res <- scdecon(bulk = bulk_sim[[1]],
#'                      ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
#'                      phenodata = phenodata,
#'                      filter_ref = TRUE,
#'                      decon_method = "OLS",
#'                      norm_method_sc = "LogNormalize",
#'                      norm_method_bulk = "TMM",
#'                      trans_method_sc = "none",
#'                      trans_method_bulk = "log2",
#'                      marker_strategy = "all")
#'
#' ## compute metrics
#' metrics_res <- compute_metrics(decon_res[[1]], bulk_sim[[2]])
#' }



compute_metrics <- function(prop_pred, prop_sim) {
  if (length(union(colnames(prop_pred), colnames(prop_sim))) != length(intersect(colnames(prop_pred), colnames(prop_sim)))) stop("different column names between prop_pred and prop_sim")
  prop_sim <- prop_sim[, match(colnames(prop_pred), colnames(prop_sim))]
  prop_pred <- prop_pred[mixedsort(rownames(prop_pred)), ]
  prop_pred <- melt(as.matrix(prop_pred))
  colnames(prop_pred) <- c("celltype", "mixture", "observed_values")
  prop_sim <- prop_sim[mixedsort(rownames(prop_sim)), ]
  prop_sim <- melt(as.matrix(prop_sim))
  colnames(prop_sim) <- c("celltype", "mixture", "expected_values")
  prop_pred$expected_values <- prop_sim$expected_values
  prop_pred$mixture_samep <- gsub("_.*", "", prop_pred$mixture)
  prop_pred_sum <- prop_pred %>%
    group_by(celltype, mixture_samep) %>%
    summarise(
      RMSE = sqrt(mean((observed_values - expected_values)^2)),
      bias = mean(observed_values - expected_values),
      variance = mean((observed_values - mean(observed_values))^2)
    )
  prop_pred_sum1 <- prop_pred %>%
    group_by(mixture) %>%
    summarise(RMSE = sqrt(mean((observed_values - expected_values)^2)))
  return(list(prop_pred_sum, prop_pred_sum1))
}

#' Methods to manipulate .gmt files
#'
#' Reformat, read & write .gmt file.
#'
#' @param gmtfile path to a gene set definition file in .gmt format.
#' @param gmt a gmt object returned by \code{read_gmt}.
#' @param outputfile full path including file name to export reformatted .gmt file.
#' @param replace a logical value indicating whether to replace the output file if it already exists. Default to FALSE.
#'
#' @return for read_gmt, returns a list object with length equal to the total number of gene sets within the .gmt file. Each list contains three elements:
#' "id", "name", "genes". read_gmt & write_gmt are reimplemented based on functions from \code{ActivePathways} package.
#'
#' @details \code{reformat_gmt} replaces blank spaces within the gene-set names to help string-matching methods in downstream plot functions \code{\link{gsea_sumplot}}
#' ,\code{\link{gsea_rwplot}}, \code{\link{gsea_heatmap}}.
#' @export
#'

reformat_gmt <- function(gmtfile, outputfile, replace = FALSE) {
  gmtinput <- read_gmt(gmtfile)
  for(i in 1:length(gmtinput)) gmtinput[[i]]$id <- gsub(" ", "_", gmtinput[[i]]$id)
  write_gmt(gmtinput, outputfile, replace = replace)
}

#' @rdname reformat_gmt
#' @export

read_gmt <- function(gmtfile) {
        gmt <- strsplit(readLines(gmtfile), "\t")
        names(gmt) <- sapply(gmt, function(i) i[1])
        gmt <- lapply(gmt, function(i) { list(id = i[1], name = i[2], genes = i[-c(1, 2)]) })
        gmt
}

#' @rdname reformat_gmt
#' @export

write_gmt <- function(gmt, outputfile, replace = FALSE) {
  if(file.exists(outputfile) && (!replace)) stop("outputfile exists!")
  sink(outputfile)
  for (term in gmt) {
      cat(term$id, term$name, paste(term$genes, collapse = "\t"), sep = "\t")
      cat("\n")
  }
  sink()
}


#' Prepare .rnk file for GSEA preranked analysis
#'
#' Prepare .rnk file for GSEA preranked analysis
#'
#' @param teststats a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
#' "p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.
#' @param outputfile full path including file name to .rnk file.
#' @param replace a logical value indicating whether to replace the output file if it already exists. Default to FALSE.
#' 
#' @return No return value
#' 
#' @export


prepare_rnk <- function(teststats, outputfile, replace = FALSE){
  if(file.exists(outputfile) && (!replace)) stop("outputfile exists!")
  if (ncol(teststats) != 5) stop("inputs need to have five columns: genename, log2foldchange, log2avgexp, pval & padj")
  colnames(teststats) <- c("genename", "log2foldchange", "log2avgexp", "pval", "padj")
  teststats <- teststats[!duplicated(teststats$genename), ]
  write.table(teststats[,c("genename", "log2foldchange")], outputfile, row.names = FALSE, col.names = FALSE, sep = "\t", quote = FALSE)
}


#' Generate a scatter plot comparing two gene set enrichment analysis results
#'
#' Generate a scatter plot of normalized enrichment scores comparing two results for gene set enrichment analysis, e.g. w/wo adjusting for cell proportion differences.
#'
#' @param gseares_path1 path to GSEA output.
#' @param gseares_path2 path to a second GSEA output.
#' @param result_names a vector of length 2 indicating the names of the two GSEA results. If NULL, names will be set to c("results1", "results2")
#' @param nes_cutoff normalized enrichment score cutoff to identify enriched gene-sets.
#' @param pval_cutoff p value cutoff to identify enriched gene-sets.
#' @param pvalflag a logical value indicating whether to use adjusted p value in selecting enriched gene-sets. Default to TRUE.
#' @param interactive a logical value indicating whether to generate an interactive plot. Default to FALSE.
#'
#' @return a \code{ggplot} object or \code{plotly} object if interactive is set to TRUE
#'
#' @export
#'
#' @details this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}
#'


comparegsea_scatter <- function(
  gseares_path1,
  gseares_path2,
  result_names = NULL,
  nes_cutoff = 2,
  pval_cutoff = 0.1,
  pvalflag = TRUE,
  interactive = FALSE){
    if ((!is.null(result_names)) && length(result_names) != 2) stop("The length of result_names has to equal to 2") else if (is.null(result_names)) result_names <- c("result1", "result2")
    gseares1 <- do.call(rbind, lapply(list.files(gseares_path1, pattern = "gsea_report_for_.*.tsv", full.names = TRUE), function(i) read.delim(i, header = TRUE)))
    gseares2 <- do.call(rbind, lapply(list.files(gseares_path2, pattern = "gsea_report_for_.*.tsv", full.names = TRUE), function(i) read.delim(i, header = TRUE)))
    gseares3 <- inner_join(gseares1, gseares2, by = "NAME")
    if (pvalflag) {
        idx1 <- which(abs(gseares3$NES.x) >= log2(nes_cutoff) & gseares3$FDR.q.val.x <= pval_cutoff)
        idx2 <- which(abs(gseares3$NES.y) >= log2(nes_cutoff) & gseares3$FDR.q.val.y <= pval_cutoff)
    } else {
        idx1 <- which(abs(gseares3$NES.x) >= log2(nes_cutoff) & gseares3$NOM.p.val.x <= pval_cutoff)
        idx2 <- which(abs(gseares3$NES.y) >= log2(nes_cutoff) & gseares3$NOM.p.val.y <= pval_cutoff)
    }
   if (length(idx1) == 0 && length(idx2) == 0) stop("No enriched genesets identified, try loosing the thresholds")
    gseares3$category <- "Not significant"
    gseares3$category[intersect(idx1, idx2)] <- "Both"
    gseares3$category[setdiff(idx1, idx2)] <- paste0(result_names[1], " only")
    gseares3$category[setdiff(idx2, idx1)] <- paste0(result_names[2], " only")
    color_df <- data.frame(
        group = c("Both", paste0(result_names[1], " only"), paste0(result_names[2], " only"), "Not significant"),
        colors = c("red", "seagreen1", "royalblue", "grey")
    )
    gseares3$category <- factor(gseares3$category, levels = color_df$group[color_df$group %in% unique(gseares3$category)])
    lim_values <- range(gseares3[, c("NES.x", "NES.y")], na.rm = TRUE)
    gp <- ggplot() +
        geom_point(aes(x = NES.x, y = NES.y, color = category, text = paste0("Geneset: ", NAME)), size = 5, data = gseares3) +
        theme_classic() + labs(x = result_names[1], y = result_names[2]) +
        scale_color_manual(values = color_df$colors[color_df$group %in% unique(gseares3$category)]) +
        coord_cartesian(xlim = lim_values, ylim = lim_values) +
        geom_hline(yintercept = c(log2(nes_cutoff), -log2(nes_cutoff)), linetype = 2) +
        geom_vline(xintercept = c(log2(nes_cutoff), -log2(nes_cutoff)), linetype = 2) +
        geom_abline(slope = 1, intercept = 0)
    if (interactive) {
        return(plotly::ggplotly(gp))
    } else {
        return(gp)
    }
}

#' Summary plot of gene set enrichment analysis
#'
#' Summary plot of selected up/down regulated gene-sets for gene set enrichment analysis.
#'
#' @param gseares_path path to GSEA output.
#' @param pos_sel a character vector of upregulated gene-set names.
#' @param neg_sel a character vector of downregulated gene-set names.
#' @param pvalflag a logical value indicating whether to use adjusted p value in the plot. Default to TRUE.
#' @param interactive a logical value indicating whether to generate an interactive plot. Default to FALSE.
#'
#' @export
#'
#' @details this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}


gsea_sumplot <- function(
  gseares_path,
  pos_sel,
  neg_sel,
  pvalflag = TRUE,
  interactive = FALSE){
  gseares_up <- read.delim(list.files(gseares_path, pattern = "gsea_report_for_.*_pos_.*.tsv", full.names = TRUE), stringsAsFactors = FALSE)
  gseares_down <- read.delim(list.files(gseares_path, pattern = "gsea_report_for_.*_neg_.*.tsv", full.names = TRUE), stringsAsFactors = FALSE)
  idx_up <- na.omit(match(pos_sel, gseares_up$NAME))
  message(paste0("found ", length(idx_up), " # of up-regulated genesets"))
  idx_down <- na.omit(match(neg_sel, gseares_down$NAME))
  message(paste0("found ", length(idx_down), " # of down-regulated genesets"))
  if(length(idx_up) != length(pos_sel) || length(idx_down) != length(neg_sel))
    stop("Please check spelling of selected gene sets")
  gseares_up <- gseares_up[idx_up,]
  gseares_down <- gseares_down[idx_down,]
  tmp <- rbind(gseares_up, gseares_down)
  tmp$NAME <- gsub("_", " ", tmp$NAME)
  if(pvalflag) pval <- "FDR.q.val" else pval <- "NOM.p.val"
  gp <- ggplot(tmp, aes(NES, NAME)) +
    geom_point(aes(colour = get(pval), size = SIZE)) + labs(colour = pval) +
    scale_color_gradientn(colours=rainbow(4), limits=c(0, 1)) +
    geom_vline(xintercept = 0, size = 0.5, colour = "gray50") +
    theme(panel.background = element_rect(fill = "gray95", colour = "gray95"),
          panel.grid.major = element_line(size = 0.25, linetype = "solid", colour = "gray90"),
          panel.grid.minor = element_line(size = 0.25,linetype = "solid", colour = "gray90"),
          axis.title.y = element_blank()) +
    scale_y_discrete(limits = rev(tmp$NAME))
  if(interactive) return(plotly::ggplotly(gp)) else return(gp)
}


#' Heatmap to demonstrate enrichment of selected gene-sets
#'
#' Heatmap to demonstrate enrichment of selected gene-sets.
#'
#' @param normdata a matrix-like object of normalized & untransformed bulk RNA-seq data, with rows representing genes and columns representing samples.
#' The first element of the output from function \code{\link{run_de}}.
#' @param teststats a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
#' "p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.
#' @param gmtfile path to gmt file used for GSEA analysis.
#' @param numgenes Number of genes to include in the heatmap. Will choose \code{numgenes} # of top up-regulated genes, as well as \code{numgenes} # of down-regulated genes
#' @param gsname_up a character value indicating selected up-regulated gene-set.
#' @param gsname_down a character value indicating selected down-regulated gene-set.
#' @param anncol a data.frame of sample meta information to include as column annotation bars. See option annCol from \code{\link[NMF]{aheatmap}} for more details.
#' @param color color used for heatmap. See option color option from \code{\link[NMF]{aheatmap}} for details.
#' @param anncolors optional data.frame to define colors for column annotations in \code{anncol}.
#' @param rankcol a logical value indicating whether to sort samples based on correlation between fold change & gene expression for better visualization. Default to TRUE.
#' @param zscore_range a vector of length two indicating the desired range of z-score transformed data. Default to c(-3, 3).
#' 
#' @return a heatmap outputted from \code{aheatmap} function from \code{NMF} package
#' 
#' @export
#'
#' @details this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}


gsea_heatmap <- function(
  normdata,
  teststats,
  gmtfile,
  numgenes,
  gsname_up,
  gsname_down,
  anncol,
  color,
  anncolors = NULL,
  rankcol = TRUE,
  zscore_range = c(-3, 3)){
  if (ncol(teststats) != 5) stop("inputs need to have five columns: genename, log2foldchange, log2avgexp, pval & padj")
  colnames(teststats) <- c("genename", "log2foldchange", "log2avgexp", "pval", "padj")
  if (length(intersect(rownames(normdata), teststats$genename)) != length(union(rownames(normdata), teststats$genename))) stop("row names of normdata not consistent with genename of teststats")
  teststats <- teststats[match(rownames(normdata), teststats$genename),]
  gmt <- read_gmt(gmtfile)
  or <- order(teststats$log2foldchange, decreasing = TRUE)
  normdata_sel <- normdata[c(or[1:numgenes], tail(or, numgenes)),]
  idx <- which(toupper(names(gmt)) == toupper(gsname_up))
  if(!length(idx)) stop(paste0(gsname_up, " not found in provided gmtfile"))
  ann_gs1 <- sapply(rownames(normdata_sel), function(i) {
    idx1 <- grep(paste0("^", i, "$"), gmt[[idx]]$genes)
    if(length(idx1)) "In GS" else "Not in GS"
  })
  idx <- which(toupper(names(gmt)) == toupper(gsname_down))
  if(!length(idx)) stop(paste0(gsname_down, " not found in provided gmtfile"))
  ann_gs2 <- sapply(rownames(normdata_sel), function(i) {
    idx1 <- grep(paste0("^", i, "$"), gmt[[idx]]$genes)
    if(length(idx1)) "In GS" else "Not in GS"
  })
  rowann <- data.frame(gsname_up = ann_gs1,
                       gsname_down = ann_gs2)
  if(is.null(anncolors)) anncolors <- list(gsname_up = c('black', 'white'), gsname_down = c('green','white'))
  tmp <- log2(normdata_sel + 0.1)
  if(rankcol) {
    colrk <- order(sapply(1:(ncol(tmp)), function(i) cor(tmp[,i], teststats$log2foldchange[c(or[1:numgenes], tail(or, numgenes))])), decreasing = FALSE)
  } else colrk <- 1:ncol(tmp)
  tmp <- t(scale(t(tmp)))
  tmp[tmp < zscore_range[1]] <- zscore_range[1]
  tmp[tmp > zscore_range[2]] <- zscore_range[2]
  NMF::aheatmap(tmp, annRow = rowann, annCol = anncol, scale = "none", labCol = NA, labRow = NA, annColors = anncolors, color = color, Rowv = NA, Colv = NA)
  }


#' GSEA random-walk plot
#'
#' Generate high-quality GSEA random-walk figures.
#'
#' @param gseares_path path to GSEA output.
#' @param gsname a character value indicating the gene-set name to generate random-walk plot.
#' @param class_name a character value indicating the class of the gene-set, e.g. "GO".
#' @param metric_range optional range of the ranking metric.
#' 
#' @return a random-walk plot for selected gene-set
#' 
#' @export
#'
#' @details this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}. Scripts initially implemented by Thomas Kuilman.
#'
#'


gsea_rwplot <- function(gseares_path, gsname, class_name, metric_range = NULL) {
  if (!dir.exists(gseares_path)) {
    stop(paste0(gsearespath, " not found"))
  }
  if (missing(gsname)) {
    stop("gsname argument is required")
  }

  ## Load .rnk data
  path_rnk <- list.files(path = file.path(gseares_path, "edb"), pattern = ".rnk$", full.names = TRUE)
  gsea_rnk <- read.delim(file = path_rnk, header = FALSE)
  colnames(gsea_rnk) <- c("hgnc.symbol", "metric")
  if (is.null(metric_range)) metric_range <- range(gsea_rnk$metric)

  ## Load .edb data
  path_edb <- list.files(path = file.path(gseares_path, "edb"), pattern = ".edb$", full.names = TRUE)
  gsea_edb <- read.delim(file = path_edb, header = FALSE, stringsAsFactors = FALSE)
  gsea_edb <- unlist(gsea_edb)
  gsea_metric <- gsea_edb[grep("METRIC=", gsea_edb)]
  gsea_metric <- unlist(strsplit(gsea_metric, " "))
  gsea_metric <- gsea_metric[grep("METRIC=", gsea_metric)]
  gsea_metric <- gsub("METRIC=", "", gsea_metric)
  gsea_edb <- gsea_edb[grep("<DTG", gsea_edb)]

  # Select the right gene set
  if (length(gsea_edb) == 0) stop(paste0("The gene set name:", gsname, " was not found, please provide a correct name"))
  if (length(grep(paste0(gsub(".\\$(.*$)", "\\1", gsname), " "), gsea_edb)) > 1) warning(paste("More than 1 gene set matched the gene.set, argument; the first match is plotted"))
  gsea_edb <- gsea_edb[grep(gsname, gsea_edb)[1]]
  # Get template name
  gsea_edb <- gsub(".*TEMPLATE=(.*)", "\\1", gsea_edb)
  gsea_edb <- unlist(strsplit(gsea_edb, " "))
  gsea_template <- gsea_edb[1]

  # Get gene set name
  gsea_gene_set <- gsea_edb[2]
  gsea_gene_set <- gsub("GENESET=gene_sets.gmt#", "", gsea_gene_set)

  # Get enrichment score
  gsea_enrichment_score <- gsea_edb[3]
  gsea_enrichment_score <- gsub("ES=", "", gsea_enrichment_score)

  # Get normalized enrichment score
  gsea_normalized_enrichment_score <- gsea_edb[4]
  gsea_normalized_enrichment_score <- gsub("NES=", "", gsea_normalized_enrichment_score)

  # Get nominal p-value
  gsea_pvalue <- gsea_edb[5]
  gsea_pvalue <- gsub("NP=", "", gsea_pvalue)
  gsea_pvalue <- as.numeric(gsea_pvalue)

  # Get FDR
  gsea_fdr <- gsea_edb[6]
  gsea_fdr <- gsub("FDR=", "", gsea_fdr)
  gsea_fdr <- as.numeric(gsea_fdr)

  # Get hit indices
  gsea_edb <- gsea_edb[grep("HIT_INDICES=", gsea_edb):length(gsea_edb)]
  gsea_hit_indices <- gsea_edb[seq_len(grep("ES_PROFILE=", gsea_edb) - 1)]
  gsea_hit_indices <- gsub("HIT_INDICES=", "", gsea_hit_indices)
  gsea_hit_indices <- as.integer(gsea_hit_indices)

  # Get ES profile
  gsea_edb <- gsea_edb[grep("ES_PROFILE=", gsea_edb):length(gsea_edb)]
  gsea_es_profile <- gsea_edb[seq_len(grep("RANK_AT_ES=", gsea_edb) - 1)]
  gsea_es_profile <- gsub("ES_PROFILE=", "", gsea_es_profile)
  gsea_es_profile <- as.numeric(gsea_es_profile)

  ## Create GSEA plot
  # Save default for resetting
  def_par <- par(no.readonly = TRUE)

  # Create a division of the device
  gsea_layout <- layout(matrix(c(1, 2, 3, 4)), heights = c(1.7, 0.5, 0.2, 2))

  # Create plots
  par(mar = c(0, 5, 2, 2))
  plot(c(1, gsea_hit_indices, length(gsea_rnk$metric)),
       c(0, gsea_es_profile, 0), type = "l", col = "red", lwd = 1.5, xaxt = "n",
       xaxs = "i", xlab = "", ylab = "Enrichment score (ES)",
       main = list(gsea_gene_set, font = 1, cex = 1),
       panel.first = {
         abline(h = seq(round(min(gsea_es_profile), digits = 1),
                        max(gsea_es_profile), 0.1),
                col = "gray95", lty = 2)
         abline(h = 0, col = "gray50", lty = 2)
       })
  plot_coordinates <- par("usr")
  if(gsea_enrichment_score < 0) {
    text(length(gsea_rnk$metric) * 0.01, plot_coordinates[3] * 0.98,
         paste("Nominal p-value:", gsea_pvalue, "\nFDR:", gsea_fdr, "\nES:",
               gsea_enrichment_score, "\nNormalized ES:",
               gsea_normalized_enrichment_score), adj = c(0, 0))
  } else {
    text(length(gsea_rnk$metric) * 0.99, plot_coordinates[4] - ((plot_coordinates[4] - plot_coordinates[3]) * 0.03),
         paste("Nominal p-value:", gsea_pvalue, "\nFDR:", gsea_fdr, "\nES:",
               gsea_enrichment_score, "\nNormalized ES:",
               gsea_normalized_enrichment_score, "\n"), adj = c(1, 1))
  }

  par(mar = c(0, 5, 0, 2))
  plot(0, type = "n", xaxt = "n", xaxs = "i", xlab = "", yaxt = "n",
       ylab = "", xlim = c(1, length(gsea_rnk$metric)))
  abline(v = gsea_hit_indices, lwd = 0.75)

  par(mar = c(0, 5, 0, 2))
  rank_colors <- gsea_rnk$metric - metric_range[1]
  rank_colors <- rank_colors / (metric_range[2] - metric_range[1])
  rank_colors <- ceiling(rank_colors * 255 + 1)
  tryCatch({
    rank_colors <- colorRampPalette(c("blue", "white", "red"))(256)[rank_colors]
  }, error = function(e) {
    stop("Please use the metric_range argument to provide a metric range that",
         "includes all metric values")
  })
  # Use rle to prevent too many objects
  rank_colors <- rle(rank_colors)
  barplot(matrix(rank_colors$lengths), col = rank_colors$values, border = NA, horiz = TRUE, xaxt = "n", xlim = c(1, length(gsea_rnk$metric)))
  box()
  text(length(gsea_rnk$metric) / 2, 0.7,
       labels = ifelse(!missing(class_name), class_name, gsea_template))
  text(length(gsea_rnk$metric) * 0.01, 0.7, "Positive", adj = c(0, NA))
  text(length(gsea_rnk$metric) * 0.99, 0.7, "Negative", adj = c(1, NA))

  par(mar = c(5, 5, 0, 2))
  rank_metric <- rle(round(gsea_rnk$metric, digits = 2))
  plot(gsea_rnk$metric, type = "n", xaxs = "i",
       xlab = "Rank in ordered gene list", xlim = c(0, length(gsea_rnk$metric)),
       ylim = metric_range, yaxs = "i",
       ylab = if(gsea_metric == "None") {"Ranking metric"} else {gsea_metric},
       panel.first = abline(h = seq(metric_range[1] / 2,
                                    metric_range[2] - metric_range[1] / 4,
                                    metric_range[2] / 2), col = "gray95", lty = 2))

  barplot(rank_metric$values, col = "lightgrey", lwd = 0.1, xaxs = "i",
          xlab = "Rank in ordered gene list", xlim = c(0, length(gsea_rnk$metric)),
          ylim = c(-1, 1), yaxs = "i", width = rank_metric$lengths, border = NA,
          ylab = ifelse(gsea_metric == "None", "Ranking metric", gsea_metric), space = 0, add = TRUE)
  box()
  # Reset to default
  on.exit(par(def_par))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{gsea_heatmap}
\alias{gsea_heatmap}
\title{Heatmap to demonstrate enrichment of selected gene-sets}
\usage{
gsea_heatmap(
  normdata,
  teststats,
  gmtfile,
  numgenes,
  gsname_up,
  gsname_down,
  anncol,
  color,
  anncolors = NULL,
  rankcol = TRUE,
  zscore_range = c(-3, 3)
)
}
\arguments{
\item{normdata}{a matrix-like object of normalized & untransformed bulk RNA-seq data, with rows representing genes and columns representing samples.
The first element of the output from function \code{\link{run_de}}.}

\item{teststats}{a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
"p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.}

\item{gmtfile}{path to gmt file used for GSEA analysis.}

\item{numgenes}{Number of genes to include in the heatmap. Will choose \code{numgenes} # of top up-regulated genes, as well as \code{numgenes} # of down-regulated genes}

\item{gsname_up}{a character value indicating selected up-regulated gene-set.}

\item{gsname_down}{a character value indicating selected down-regulated gene-set.}

\item{anncol}{a data.frame of sample meta information to include as column annotation bars. See option annCol from \code{\link[NMF]{aheatmap}} for more details.}

\item{color}{color used for heatmap. See option color option from \code{\link[NMF]{aheatmap}} for details.}

\item{anncolors}{optional data.frame to define colors for column annotations in \code{anncol}.}

\item{rankcol}{a logical value indicating whether to sort samples based on correlation between fold change & gene expression for better visualization. Default to TRUE.}

\item{zscore_range}{a vector of length two indicating the desired range of z-score transformed data. Default to c(-3, 3).}
}
\value{
a heatmap outputted from \code{aheatmap} function from \code{NMF} package
}
\description{
Heatmap to demonstrate enrichment of selected gene-sets.
}
\details{
this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{comparegsea_scatter}
\alias{comparegsea_scatter}
\title{Generate a scatter plot comparing two gene set enrichment analysis results}
\usage{
comparegsea_scatter(
  gseares_path1,
  gseares_path2,
  result_names = NULL,
  nes_cutoff = 2,
  pval_cutoff = 0.1,
  pvalflag = TRUE,
  interactive = FALSE
)
}
\arguments{
\item{gseares_path1}{path to GSEA output.}

\item{gseares_path2}{path to a second GSEA output.}

\item{result_names}{a vector of length 2 indicating the names of the two GSEA results. If NULL, names will be set to c("results1", "results2")}

\item{nes_cutoff}{normalized enrichment score cutoff to identify enriched gene-sets.}

\item{pval_cutoff}{p value cutoff to identify enriched gene-sets.}

\item{pvalflag}{a logical value indicating whether to use adjusted p value in selecting enriched gene-sets. Default to TRUE.}

\item{interactive}{a logical value indicating whether to generate an interactive plot. Default to FALSE.}
}
\value{
a \code{ggplot} object or \code{plotly} object if interactive is set to TRUE
}
\description{
Generate a scatter plot of normalized enrichment scores comparing two results for gene set enrichment analysis, e.g. w/wo adjusting for cell proportion differences.
}
\details{
this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvolution.R
\name{scdecon}
\alias{scdecon}
\title{Deconvolution of bulk RNA-seq data}
\usage{
scdecon(
  bulk,
  ref,
  phenodata,
  filter_ref = TRUE,
  marker_genes = NULL,
  genes_to_remove = NULL,
  min_pct_ct = 0.05,
  decon_method = c("scaden", "CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR", "MuSiC",
    "SCDC", "scTAPE"),
  norm_method_sc = c("none", "LogNormalize", "SCTransform", "scran", "scater", "Linnorm"),
  norm_method_bulk = c("none", "TMM", "median_ratios", "TPM"),
  trans_method_sc = c("none", "log2", "sqrt", "vst"),
  trans_method_bulk = c("none", "log2", "sqrt", "vst"),
  gene_length = NULL,
  lfc_markers = log2(1.5),
  marker_strategy = c("all", "pos_fc", "top_50p_logFC", "top_50p_AveExpr"),
  to_remove = NULL,
  ffpe_artifacts = FALSE,
  model = NULL,
  prop = NULL,
  cibersortpath = NULL,
  pythonpath = NULL,
  tmpdir = NULL,
  remove_tmpdir = TRUE,
  seed = 1234,
  nsamples = 1000,
  return_value_only = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{bulk}{a matrix or data.frame of unnormalizaed & untransformed bulk RNA-seq gene expression values with rows representing genes, columns
representing samples}

\item{ref}{a matrix or data.frame of untransformed scRNA-seq gene expression counts with rows representing genes, columns representing cells.
This data will be used to deconvolute provided bulk RNA-seq data.}

\item{phenodata}{a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first three
columns as:
\enumerate{
 \item cell barcodes
 \item cell types
 \item subject ids
}}

\item{filter_ref}{logical value indicating whether outlier genes & cells should be removed from the provided reference data. Defaults to TRUE}

\item{marker_genes}{a data.frame of two columns. First column represents cell types in ref; second column represents gene names of marker genes. If specified,
those genes will be used to construct signature matrix for mark-gene based deconvolution methods, such as CIBERSORT, OLS, nnls, FARDEEP and RLR. Default to NULL,
carry out differential analysis to identify marker genes for each cell type in ref.}

\item{genes_to_remove}{a vector of gene names to remove from the reference scRNA-seq data. Default to NULL.}

\item{min_pct_ct}{a numeric value indicating the minimum required proportion of expressing cells per cell type for marker gene identification. Only applicable when marker_genes
is NULL. Default to 0.05.}

\item{decon_method}{character value specifying the deconvolution method to use. Has to be one of "scaden", "CIBERSORT", "OLS", "nnls", "FARDEEP", "RLR",
"MuSiC", "SCDC", "scTAPE". See details for more information.}

\item{norm_method_sc}{character value specifying the normalization method to use for reference data. Has to be one of "none","LogNormalize",
"SCTransform", "scran", "scater", "Linnorm". See details for more information.}

\item{norm_method_bulk}{character value specifying the normalization method to use for bulk data. Has to be one of "none", "TMM",
"median_ratios", "TPM". See details for more information.}

\item{trans_method_sc}{character value specifying the transformation method to use for both bulk & reference data. Has to be one of "none", "log2", "sqrt",
"vst". See details for more information.}

\item{trans_method_bulk}{character value specifying the transformation method to use for both bulk & reference data. Has to be one of "none", "log2", "sqrt",
"vst". See details for more information.}

\item{gene_length}{a data.frame with two columns. The first column represents gene names that match with provided bulk data. The second column
represents length of each gene. Only applicable when norm_method is selected as "TPM".}

\item{lfc_markers}{log2 fold change cutoff used to identify marker genes for deconvolution. The option only applicable to marker-gene based
approaches, such as CIBERSORT, OLS, nnls, FARDEEP and RLR. Only applicable when marker_genes is NULL.}

\item{marker_strategy}{further strategy in selecting marker genes besides applying the log2 fold change cutoff. Can be chosen from: "all", "pos_fc",
"top_50p_logFC" or "top_50p_AveExpr". See details for more information. Only applicable when marker_genes is NULL.}

\item{to_remove}{character value representing the cell type to remove from reference data. Only applicable to simulation experiments in evaluating
effect of cell type removal from reference.}

\item{ffpe_artifacts}{logical value indicating whether to add simulated ffpe artifacts in the bulk data. Only applicable to simulation experiments in
evaluating the effect of FFPE artifacts.}

\item{model}{pre-constructed ffpe model data. Can be downloaded from github: https://github.com/Liuy12/SCdeconR_files/blob/master/data/ffpemodel.rda.}

\item{prop}{a matrix or data.frame of simulated cell proportion values with rows representing cell types, columns representing samples. Only applicable to simulation
experiments in evaluating the effect of cell type removal from reference.}

\item{cibersortpath}{full path to CIBERSORT.R script.}

\item{pythonpath}{full path to python binary where scaden was installed with.}

\item{tmpdir}{temporary processing directory for scaden or scTAPE.}

\item{remove_tmpdir}{a logical value indicating whether to remove tmpdir once scaden is completed. Default to TRUE.}

\item{seed}{random seed used for simulating FFPE artifacts. Only applicable when ffpe_artifacts is set to TRUE.}

\item{nsamples}{number of artificial bulk samples to simulate for scaden. Default to 1000.}

\item{return_value_only}{return a list of values only without performing deconvolution. This could be helpful in
cases where the user want to apply their own deconvolution algorithms. Default to FALSE.}

\item{verbose}{a logical value indicating whether to print messages. Default to FALSE.}
}
\value{
a list containing two or four elements.
\describe{
 \item{first element}{a data.frame of predicted cell-type proportions, with rows representing cell types, columns representing samples.}
 \item{second element}{a data.frame of fitting errors of the algorithm; first column represents sample names, second column represents RMSEs.}
 \item{optional third element}{a data.frame of simulated cell proportion after removing the specified cell_type. Only applicable to simulation experiments.}
 \item{optional fourth element}{a data.frame of marker genes used for deconvolution. Only applicable to marker-gene based deconvolution methods.}
}
}
\description{
Deconvolution of bulk RNA-seq data based on single-cell reference data. Eight bulk deconvolution methods, along with eight normalization
methods and four transformation methods are available.
}
\details{
decon_method should be one of the following:
\describe{
 \item{\href{https://github.com/KevinMenden/scaden}{scaden}}{a deep learning based method using three multi-layer deep neural nets. To use scaden,
you need to firstly install scaden via \code{pip} or \code{conda}, the provide the python path to \code{pythonpath} option.}
 \item{CIBERSORT}{a marker gene based support vectors regression approach. CIBERSOR does not allow redistribution. To use CIBERSORT,
you need to request the source code from the authors & provide the path of CIBERSORT.R script to \code{cibersortpath} option.}
 \item{OLS}{ordinary least squares.}
 \item{\href{https://CRAN.R-project.org/package=nnls}{nnls}}{non-negative least squares.}
 \item{\href{https://CRAN.R-project.org/package=FARDEEP}{FARDEEP}}{robust regression using least trimmed squares}
 \item{\href{https://CRAN.R-project.org/package=MASS}{RLR}}{robust regression using an M estimator}
 \item{\href{https://github.com/xuranw/MuSiC}{MuSiC}}{multi-subject single-cell deconvolution}
 \item{\href{https://github.com/meichendong/SCDC}{SCDC}}{an ENSEMBLE method to integrate deconvolution results from different scRNA-seq datasets}
 \item{\href{https://github.com/poseidonchan/TAPE}{scTAPE}}{Deep autoencoder based deconvolution}
}

norm_method should be one of the following:
\describe{
 \item{none}{no normalization is performed.}
 \item{LogNormalize}{\code{\link[Seurat]{LogNormalize}} method from seurat.}
 \item{TMM}{TMM method from \code{\link[edgeR]{calcNormFactors}} function from edgeR.}
 \item{median_ratios}{median ratio method from \code{\link[DESeq2]{estimateSizeFactors,DESeqDataSet-method}} function from DESeq2.}
 \item{TPM}{Transcript per million. TPM has to be chosen if ffpe_artifacts is set to TRUE.}
 \item{SCTransform}{\code{\link[Seurat]{SCTransform}} method from Seurat.}
 \item{scran}{\code{\link[scran]{computeSumFactors}} method from scran.}
 \item{scater}{\code{\link[scater]{librarySizeFactors}} method from scater.}
 \item{Linnorm}{\code{Linnorm} method from Linnorm.}
}

trans_method should be one of the following:
\describe{
 \item{none}{no transformation is performed.}
 \item{log2}{log2 transformation. 0.1 is added to the data to avoid logarithm of 0s.}
 \item{sqrt}{square root transformation.}
 \item{vst}{\code{\link[DESeq2]{varianceStabilizingTransformation}} method from DESeq2.}
}

marker_strategy should be one of the following:
\describe{
 \item{all}{all genes passed fold change threshold will be used.}
 \item{pos_fc}{only genes with positive fold changes will be used.}
 \item{top_50p_logFC}{only genes with top 50 percent positive fold changes will be used.}
 \item{top_50p_AveExpr}{only genes with top 50 percent average expression will be used.}
}
}
\examples{
\dontrun{
ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
              paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)

# construct integrated reference using harmony algorithm
refdata <- construct_ref(ref_list = ref_list,
                      phenodata_list = phenodata_list,
                      data_type = "cellranger",
                      method = "harmony",
                      group_var = "subjectid",
                      nfeature_rna = 50,
                      vars_to_regress = "percent_mt", verbose = FALSE)

phenodata <- data.frame(cellid = colnames(refdata),
                        celltypes = refdata$celltype,
                        subjectid = refdata$subjectid)
bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                           phenodata = phenodata,
                           num_mixtures = 20,
                           num_mixtures_sprop = 1)

## perform deconvolution based on "OLS" algorithm
decon_res <- scdecon(bulk = bulk_sim[[1]],
                     ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                     phenodata = phenodata,
                     filter_ref = TRUE,
                     decon_method = "OLS",
                     norm_method_sc = "LogNormalize",
                     norm_method_bulk = "TMM",
                     trans_method_sc = "none",
                     trans_method_bulk = "log2",
                     marker_strategy = "all")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{run_de}
\alias{run_de}
\title{Differential expression analysis}
\usage{
run_de(
  bulk,
  prop = NULL,
  sampleinfo,
  control = NULL,
  case = NULL,
  de_method = c("edgeR", "DESeq2", "limma_voom", "limma"),
  padj_method = "BH",
  ...
)
}
\arguments{
\item{bulk}{a matrix-like object of gene expression values with rows representing genes, columns representing samples}

\item{prop}{a matrix-like object of cell proportion values with rows representing cell types, columns representing samples. Default to NULL,
not adjust for cell proportion.}

\item{sampleinfo}{a data.frame of metadata for the samples. Rows represents samples; columns represents covariates to adjust for. The first column of
sampleinfo should contains group information for differential analysis.}

\item{control}{a character value indicating the control group in sampleinfo. Set to NULL to perform ANOVA-like analysis.}

\item{case}{a character value indicating the case group in sampleinfo. Set to NULL to perform ANOVA-like analysis.}

\item{de_method}{a character value indicating the method to use for testing differential expression. Should be one of "edgeR", "DESeq2", "limma_voom", "limma"}

\item{padj_method}{method for adjusting multiple hypothesis testing. Default to "BH". See \code{\link{p.adjust}} for more details.}

\item{...}{parameters pass to DE methods.}
}
\value{
a list of two elements:
\enumerate{
 \item a data.frame containing normalized gene expression data.
 \item a data.frame containing detailed differential expression statistics. Columns represent "Gene name", "log2 fold change", "log2 average expression",
"p value", "adjusted p value" respectively
}
}
\description{
Performing differential expression analysis adjusting for cell proportion differences, and other covariates using a additive model.
}
\details{
To perform ANOVA like analysis (differences between any groups), set \code{control} & \code{case} options to \code{NULL} and choose one of the following methods:
edgeR, limma_voom or limma. DESeq2 does not provide direct support for this type of comparison.
}
\examples{
\dontrun{
ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
              paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)

# construct integrated reference using harmony algorithm
refdata <- construct_ref(ref_list = ref_list,
                      phenodata_list = phenodata_list,
                      data_type = "cellranger",
                      method = "harmony",
                      group_var = "subjectid",
                      nfeature_rna = 50,
                      vars_to_regress = "percent_mt", verbose = FALSE)
phenodata <- data.frame(cellid = colnames(refdata),
                        celltypes = refdata$celltype,
                        subjectid = refdata$subjectid)
## construct a vector with same proportions across cell types
prop1 <- data.frame(celltypes = unique(refdata$celltype),
                   proportion = rep(0.125, 8))
## simulate 20 bulk samples based on specified cell type proportion
bulk_sim1 <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                            phenodata = phenodata,
                            num_mixtures = 20,
                            prop = prop1, replace = TRUE)
## generate another vector with high proportion for a certian cell type
prop2 <- data.frame(celltypes = unique(refdata$celltype),
                    proportion = c(0.8, 0.1, 0.1, rep(0, 5)))
bulk_sim2 <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                            phenodata = phenodata,
                            num_mixtures = 20,
                            prop = prop2, replace = TRUE)
## compare data for differential analysis
bulk_sim <- list(cbind(bulk_sim1[[1]], bulk_sim2[[1]]),
                 cbind(bulk_sim1[[2]], bulk_sim2[[2]]))
## force to be integer for DE purposes
bulk <- round(bulk_sim[[1]], digits=0)
colnames(bulk) <- paste0("sample", 1:ncol(bulk))
## predict cell type proportions using "OLS" algorithm
decon_res <- scdecon(bulk = bulk,
                     ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                     phenodata = phenodata,
                     filter_ref = TRUE,
                     norm_method_sc = "LogNormalize",
                     norm_method_bulk = "TMM",
                     trans_method_sc = "none",
                     trans_method_bulk = "log2",
                     marker_strategy = "all")
## create sampleinfo
sampleinfo <- data.frame(condition = rep(c("group1", "group2"), each =20))
row.names(sampleinfo) <- colnames(bulk)
library(DESeq2)
deres <- run_de(bulk = bulk,
               prop = decon_res[[1]],
               sampleinfo = sampleinfo,
               control = "group1",
               case = "group2",
               de_method = "edgeR")

 ## run differential analysis without adjusting for cell proportion differences
 deres_notadjust <- run_de(bulk = bulk,
                           prop = NULL,
                           sampleinfo = sampleinfo,
                           control = "group1",
                           case = "group2",
                           de_method = "edgeR")

## scatter plot to compare the effect of adjusting cell proportion differences
comparedeg_scatter(results1 = deres[[2]],
                   results2 = deres_notadjust[[2]],
                   result_names = c("adjust for cell proportion", "not adjust for cell proportion"),
                   fc_cutoff = 1.5,
                   pval_cutoff = 0.05,
                   pvalflag = TRUE,
                   interactive = T)

## generate cell-type specific gene expression
ct_exprs_list <- celltype_expression(bulk = bulk,
                                     ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                                     phenodata = phenodata,
                                     prop = decon_res[[1]],
                                     UMI_min = 0,
                                     CELL_MIN_INSTANCE = 1)


}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{gsea_sumplot}
\alias{gsea_sumplot}
\title{Summary plot of gene set enrichment analysis}
\usage{
gsea_sumplot(
  gseares_path,
  pos_sel,
  neg_sel,
  pvalflag = TRUE,
  interactive = FALSE
)
}
\arguments{
\item{gseares_path}{path to GSEA output.}

\item{pos_sel}{a character vector of upregulated gene-set names.}

\item{neg_sel}{a character vector of downregulated gene-set names.}

\item{pvalflag}{a logical value indicating whether to use adjusted p value in the plot. Default to TRUE.}

\item{interactive}{a logical value indicating whether to generate an interactive plot. Default to FALSE.}
}
\description{
Summary plot of selected up/down regulated gene-sets for gene set enrichment analysis.
}
\details{
this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{comparedeg_scatter}
\alias{comparedeg_scatter}
\title{Generate a scatter plot comparing two differential expression results}
\usage{
comparedeg_scatter(
  results1,
  results2,
  result_names = NULL,
  fc_cutoff,
  pval_cutoff,
  pvalflag = TRUE,
  interactive = FALSE
)
}
\arguments{
\item{results1}{a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
"p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.}

\item{results2}{similar to \code{results1}.}

\item{result_names}{a vector of length 2 indicating the names of the two differential results. If NULL, names will be set to c("results1", "results2")}

\item{fc_cutoff}{fold change cutoff to identify differential expressed genes.}

\item{pval_cutoff}{p value cutoff to identify differential expressed genes.}

\item{pvalflag}{a logical value indicating whether to use adjusted p value in selecting differential expressed genes.}

\item{interactive}{a logical value indicating whether to generate an interactive plot.}
}
\description{
Generate a scatter plot of fold changes comparing two differential expression results, e.g. w/wo adjusting for cell proportion differences.
}
\details{
See examples from \code{\link{run_de}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{reformat_gmt}
\alias{reformat_gmt}
\alias{read_gmt}
\alias{write_gmt}
\title{Methods to manipulate .gmt files}
\usage{
reformat_gmt(gmtfile, outputfile, replace = FALSE)

read_gmt(gmtfile)

write_gmt(gmt, outputfile, replace = FALSE)
}
\arguments{
\item{gmtfile}{path to a gene set definition file in .gmt format.}

\item{outputfile}{full path including file name to export reformatted .gmt file.}

\item{replace}{a logical value indicating whether to replace the output file if it already exists. Default to FALSE.}

\item{gmt}{a gmt object returned by \code{read_gmt}.}
}
\value{
for read_gmt, returns a list object with length equal to the total number of gene sets within the .gmt file. Each list contains three elements:
"id", "name", "genes". read_gmt & write_gmt are reimplemented based on functions from \code{ActivePathways} package.
}
\description{
Reformat, read & write .gmt file.
}
\details{
\code{reformat_gmt} replaces blank spaces within the gene-set names to help string-matching methods in downstream plot functions \code{\link{gsea_sumplot}}
,\code{\link{gsea_rwplot}}, \code{\link{gsea_heatmap}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{transformation}
\alias{transformation}
\title{Transformation of gene expression data}
\usage{
transformation(matrix, option)
}
\arguments{
\item{matrix}{a matrix-like objector of gene expression values with rows representing genes, columns representing samples or cells}

\item{option}{character value specifying the transformation method to use. Has to be one of "none", "log", "sqrt", "vst".}
}
\value{
a matrix-like object with the same dimension of input object after data transformation.
}
\description{
Methods to use for data transformation.
}
\details{
refer to \code{\link{scdecon}} for more details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_bulk.R
\name{compute_metrics}
\alias{compute_metrics}
\title{Statistical evaluations of predicted cell proportions}
\usage{
compute_metrics(prop_pred, prop_sim)
}
\arguments{
\item{prop_pred}{a matrix-like object of predicted cell proportion values with rows representing cell types, columns representing samples.}

\item{prop_sim}{a matrix-like object of simulated/expected cell proportion values with rows representing cell types, columns representing samples.}
}
\value{
a list of two objects:
\enumerate{
 \item a data.fame of summary metrics containing RMSE, bias & variance grouped by cell types and mixture ids (simulated samples with the same expected cell proportions).
 \item a data.frame of aggregated RMSE values across all cell types within each sample.
}
}
\description{
Compute RMSE, bias & variance metrics for predicted cell proportions by comparing with expected cell proportions.
}
\examples{
\donttest{
## generate artificial bulk samples
ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
              paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)

# construct integrated reference using harmony algorithm
refdata <- construct_ref(ref_list = ref_list,
                      phenodata_list = phenodata_list,
                      data_type = "cellranger",
                      method = "harmony",
                      group_var = "subjectid",
                      nfeature_rna = 50,
                      vars_to_regress = "percent_mt", verbose = FALSE)
phenodata <- data.frame(cellid = colnames(refdata),
                        celltypes = refdata$celltype,
                        subjectid = refdata$subjectid)
prop <- data.frame(celltypes = unique(refdata$celltype), 
proportion = rep(1/length(unique(refdata$celltype)), length(unique(refdata$celltype))))   
bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                           phenodata = phenodata,
                           num_mixtures = 20,
                           prop = prop,
                           num_mixtures_sprop = 1)

## perform deconvolution based on "OLS" algorithm
decon_res <- scdecon(bulk = bulk_sim[[1]],
                     ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                     phenodata = phenodata,
                     filter_ref = TRUE,
                     decon_method = "OLS",
                     norm_method_sc = "LogNormalize",
                     norm_method_bulk = "TMM",
                     trans_method_sc = "none",
                     trans_method_bulk = "log2",
                     marker_strategy = "all")

## compute metrics
metrics_res <- compute_metrics(decon_res[[1]], bulk_sim[[2]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{prepare_rnk}
\alias{prepare_rnk}
\title{Prepare .rnk file for GSEA preranked analysis}
\usage{
prepare_rnk(teststats, outputfile, replace = FALSE)
}
\arguments{
\item{teststats}{a data.frame containing differential expression results with five columns: "Gene name", "log2 fold change", "log2 average expression",
"p value", "adjusted p value". The second element of the output from function \code{\link{run_de}}.}

\item{outputfile}{full path including file name to .rnk file.}

\item{replace}{a logical value indicating whether to replace the output file if it already exists. Default to FALSE.}
}
\value{
No return value
}
\description{
Prepare .rnk file for GSEA preranked analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reference_construction.R
\name{load_scdata}
\alias{load_scdata}
\title{Load, filter and normalize scRNA-seq/snRNA-seq data}
\usage{
load_scdata(
  ref,
  data_type = c("cellranger", "h5", "matrix"),
  meta_info,
  nfeature_rna = 200,
  percent_mt = 40,
  cc.genes = NULL,
  vars_to_regress = c("percent_mt", "phase"),
  id,
  verbose,
  ...
)
}
\arguments{
\item{ref}{path to scRNA-seq/snRNA-seq data.}

\item{data_type}{a character value specifying data type of the input scRNA-seq/snRNA-seq data, should be one of "cellranger", "h5", "matrix".}

\item{meta_info}{a data.frame with rows representing cells, columns representing cell attributes.}

\item{nfeature_rna}{minimum # of features with non-zero UMIs. Cells with # of features lower than nfeature_rna will be removed. Default to 200.}

\item{percent_mt}{maximum percentage of mitochondria (MT) mapped UMIs. Cells with MT percentage higher than percent_mt will be removed. Default to 40.}

\item{cc.genes}{cell-cycle genes curated by Seurat. Can be loaded via \code{data(cc.genes)}}

\item{vars_to_regress}{a list of character values indicating the variables to regress for SCTransform normalization step. Default is to regress
out MT percentage ("percent_mt") & cell cycle effects ("phase")}

\item{id}{a character value specifying project or sample id. Only used for printing purposes.}

\item{verbose}{logical value indicating whether to print messages.}

\item{...}{additional parameters passed to \code{\link[Seurat]{SCTransform}}.}
}
\value{
a \code{\link[Seurat]{Seurat-class}} object.
}
\description{
Load and preprocess scRNA-seq/snRNA-seq data using seurat SCTransform workflow.
}
\details{
For more details, refer to \code{\link{construct_ref}}
}
\examples{
\donttest{
samplepath1 <- paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1")
samplepath2 <- paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2")
ref_list <- c(samplepath1, samplepath2)
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)
tmp <- load_scdata(
  ref = ref_list[[1]],
  data_type = c("cellranger"),
  meta_info = data.table::fread(file = phenodata_list[[1]], check.names = FALSE, header = TRUE),
  nfeature_rna = 50,
  vars_to_regress = c("percent_mt"),
  id = 1,
  verbose = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reference_construction.R
\name{construct_ref}
\alias{construct_ref}
\title{Integration of single-cell/nuclei RNA-seq data as reference}
\usage{
construct_ref(
  ref_list,
  phenodata_list,
  data_type = c("cellranger", "h5", "matrix"),
  method = c("harmony", "seurat"),
  group_var,
  nfeature_rna = 200,
  percent_mt = 40,
  vars_to_regress = c("percent_mt", "phase"),
  ex_features = NULL,
  cluster = TRUE,
  resolution = 0.8,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{ref_list}{a character vector of data paths to scRNA-seq/snRNA-seq. See \code{data_type} for accepted data types.}

\item{phenodata_list}{a character vector of data paths to metadata for elements in ref_list. All metadata within phenodata_list should have consistent column names. Columns represent
cell attributes, such as cell type, rows represent cells. Each element in \code{phenodata_list} should at least contain the first two columns as:
\enumerate{
 \item cell barcodes
 \item cell types
}}

\item{data_type}{data type of the input scRNA-seq/snRNA-seq data. Could be either a single character value from "cellranger", "h5", "matrix", or
a vector/list of values with the same length as ref_list indicating the data type for each element.}

\item{method}{character value specifying the method to use. Has to be one of "harmony" or "seurat". See details for more information.}

\item{group_var}{a vector of character values indicating which variables within phenodata_list metadata to use for integration. Only applicable when method
is set to "harmony".}

\item{nfeature_rna}{minimum # of features with non-zero UMIs. Cells with # of features lower than nfeature_rna will be removed. Default to 200.}

\item{percent_mt}{maximum percentage of mitochondria (MT) mapped UMIs. Cells with MT percentage higher than percent_mt will be removed. Default to 40.}

\item{vars_to_regress}{a list of character values indicating the variables to regress for SCTransform normalization step. Default is to regress
out MT percentage ("percent_mt") & cell cycle effects ("phase")}

\item{ex_features}{a vector of character values indicating genes to exclude from anchor features. Those genes will not be considered as
anchor features for integration, but will still be present in the integrated data.}

\item{cluster}{logical value indicating whether to perform clustering on the integrated data. If TRUE, unsupervised clustering will be performed, and
the results will be saved in "seurat_clusters" metadata in the output Seurat object.}

\item{resolution}{numeric value specifying resolution to use when cluster is set to TRUE.}

\item{verbose}{logical value indicating whether to print messages.}

\item{...}{additional parameters passed to \code{\link[Seurat]{SCTransform}}.}
}
\value{
a \code{\link[Seurat]{Seurat-class}} object.
}
\description{
Integration of scRNA-seq or snRNA-seq data using either harmony or seurat.
}
\details{
data_type can be chosen from:
\describe{
 \item{cellranger}{path to a directory containing the matrix.mtx, genes.tsv (or features.tsv), and barcodes.tsv files outputted by 10x's cell-ranger}
 \item{h5}{path to .h5 file outputted by 10x's cell-ranger}
 \item{matrix}{path to a matrix-like file, with rows representing genes, columns representing cells.}
}

SCTransform with \code{vst.flavor = "v2"} is used for normalization of individual data. Integration methods can be chosen from either "harmony"
or "seurat". Harmony typically is more memory efficient and, recommended if you have large # of cells for integration.
}
\examples{
\donttest{
## random subset of two scRNA-seq datasets for breast tissue
ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
              paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)

## Register backend for parallel processing
#registerDoFuture()
#plan("multisession", workers = 4)

## construct integrated reference data
refdata <- construct_ref(ref_list = ref_list,
                         phenodata_list = phenodata_list,
                         data_type = "cellranger",
                         method = "harmony",
                         group_var = "subjectid",
                         nfeature_rna = 50,
                         vars_to_regress = "percent_mt")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SCdeconR-package.R
\docType{package}
\name{SCdeconR-package}
\alias{SCdeconR}
\alias{SCdeconR-package}
\title{SCdeconR: Deconvolution of Bulk RNA-Seq Data using Single-Cell RNA-Seq Data as Reference}
\description{
Streamlined workflow from deconvolution of bulk RNA-seq data to downstream differential expression and gene-set enrichment analysis. Provide various visualization functions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Liuy12/SCdeconR/}
  \item \url{https://liuy12.github.io/SCdeconR/}
}

}
\author{
\strong{Maintainer}: Yuanhang Liu \email{liu.yuanhang@mayo.edu}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_bulk.R
\name{bulk_generator}
\alias{bulk_generator}
\title{Generate artificial bulk RNA-seq samples based on simulation}
\usage{
bulk_generator(
  ref,
  phenodata,
  num_mixtures = 500,
  num_mixtures_sprop = 10,
  pool_size = 100,
  seed = 1234,
  prop = NULL,
  replace = FALSE
)
}
\arguments{
\item{ref}{a matrix-like object of gene expression values with rows representing genes, columns representing cells.}

\item{phenodata}{a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first two columns as:
\enumerate{
 \item cell barcodes
 \item cell types
}}

\item{num_mixtures}{total number of simulated bulk samples. Have to be multiple of \code{num_mixtures_sprop}. Default to 500.}

\item{num_mixtures_sprop}{number of simulated bulk samples with the same simulated cell type proportions. Only applicable when \code{prop} is not specified.
Those samples will be used to estimate bias & variance. Default to 10.}

\item{pool_size}{number of cells to use to construct each artificial bulk sample. Default to 100.}

\item{seed}{seed to use for simulation. Default to 1234.}

\item{prop}{a data.frame with two columns. The first column includes unique cell types in phenodata; the second column includes cell type proportions.
If specified, bulk samples will be simulated based on the specified cell proportions.}

\item{replace}{logical value indicating whether to sample cells with replacement. Default to FALSE, to sample cells without replacement.}
}
\value{
a list of two objects:
\enumerate{
 \item simulated bulk RNA-seq data, with rows representing genes, columns representing samples
 \item cell type proportions used to simulate the bulk RNA-seq data, with rows representing cell types, columns representing samples
}
}
\description{
Generate artificial bulk RNA-seq samples with random or pre-defined cell-type proportions for benchmarking deconvolution algorithms
}
\details{
If \code{prop} is not specified, cell type proportions will be firstly randomly generated with at least two cell types present. Then, for each cell proportion
vector, \code{num_mixtures_sprop} number of samples is simulated. Eventually, a total of \code{num_mixtures} number of samples is simulated. If prop is
specified, then a total of \code{num_mixtures} number of samples will be simulated based on the same cell proportion vector specified.
}
\examples{
\donttest{
ref_list <- c(paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample1"),
              paste0(system.file("extdata", package = "SCdeconR"), "/refdata/sample2"))
phenopath1 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample1.txt")
phenopath2 <- paste0(system.file("extdata", package = "SCdeconR"),
"/refdata/phenodata_sample2.txt")
phenodata_list <- c(phenopath1,phenopath2)

# construct integrated reference using harmony algorithm
refdata <- construct_ref(ref_list = ref_list,
                      phenodata_list = phenodata_list,
                      data_type = "cellranger",
                      method = "harmony",
                      group_var = "subjectid",
                      nfeature_rna = 50,
                      vars_to_regress = "percent_mt", verbose = FALSE)
phenodata <- data.frame(cellid = colnames(refdata),
                        celltypes = refdata$celltype,
                        subjectid = refdata$subjectid)
prop <- data.frame(celltypes = unique(refdata$celltype), 
proportion = rep(1/length(unique(refdata$celltype)), length(unique(refdata$celltype))))
bulk_sim <- bulk_generator(ref = GetAssayData(refdata, layer = "data", assay = "SCT"),
                           phenodata = phenodata,
                           num_mixtures = 20,
                           prop = prop,
                           num_mixtures_sprop = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{scaling}
\alias{scaling}
\title{Normalization of gene expression data}
\usage{
scaling(
  matrix,
  option,
  gene_length = NULL,
  seed = 1234,
  ffpe_artifacts = FALSE,
  model = NULL
)
}
\arguments{
\item{matrix}{a matrix-like objector of gene expression values with rows representing genes, columns representing samples or cells}

\item{option}{character value specifying the normalization method to use. Has to be one of "none", "LogNormalize", "TMM", "median_ratios", "TPM",
"SCTransform", "scran", "scater", "Linnorm".}

\item{gene_length}{a data.frame with two columns. The first column represents gene names that match with provided bulk data. The second column
represents length of each gene. Only applicable when norm_method is selected as "TPM"}

\item{seed}{random seed used for simulating FFPE artifacts. Only applicable when ffpe_artifacts is set to TRUE.}

\item{ffpe_artifacts}{logical value indicating whether to add simulated ffpe artifacts in the bulk data. Only applicable to simulation experiments in
evaluating the effect of FFPE artifacts.}

\item{model}{pre-constructed ffpe model data. Can be downloaded from github: https://github.com/Liuy12/SCdeconR_files/blob/master/data/ffpemodel.rda}
}
\value{
a matrix-like object with the same dimension of input object after data normalization.
}
\description{
Methods to use for data normalization.
}
\details{
refer to \code{\link{scdecon}} for more details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvolution.R
\name{prop_barplot}
\alias{prop_barplot}
\title{Bar plot of cell type proportions across samples}
\usage{
prop_barplot(prop, sort = TRUE, interactive = FALSE)
}
\arguments{
\item{prop}{a matrix or data.frame of cell proportion values with rows representing cell types, columns representing samples.}

\item{sort}{a logical value indicating whether to sort the samples based on cell type with highest median cell proportion across samples. Default to TRUE.}

\item{interactive}{a logical value indicating whether to generate interactive plot. Default to FALSE.}
}
\description{
Bar plot of cell type proportions across samples
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differential_expression.R
\name{celltype_expression}
\alias{celltype_expression}
\title{Compute cell type specific gene expression}
\usage{
celltype_expression(bulk, ref, phenodata, prop, ...)
}
\arguments{
\item{bulk}{a matrix-like object of bulk RNA-seq data with rows representing genes, columns representing samples}

\item{ref}{a matrix-like object of scRNA-seq data with rows representing genes, columns representing cells.}

\item{phenodata}{a data.frame with rows representing cells, columns representing cell attributes. It should at least contain the first two
columns as:
\enumerate{
 \item cell barcodes
 \item cell types
}}

\item{prop}{a matrix-like object of cell proportion values with rows representing cell types, columns representing samples.}

\item{...}{additional parameters passed to \code{create.RCTD} from \code{spacexr}.}
}
\value{
a list with length equal to number of unique cell types in phenodata. Each element in the list represents gene expression matrix for each unique cell type.
}
\description{
Compute cell type specific gene expression based on predicted cell proportions and reference data.
}
\details{
this function is inspired by cell-type specific gene expression estimation for doublet mode in \code{spacexr}. See examples from \code{\link{run_de}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_analysis.R
\name{gsea_rwplot}
\alias{gsea_rwplot}
\title{GSEA random-walk plot}
\usage{
gsea_rwplot(gseares_path, gsname, class_name, metric_range = NULL)
}
\arguments{
\item{gseares_path}{path to GSEA output.}

\item{gsname}{a character value indicating the gene-set name to generate random-walk plot.}

\item{class_name}{a character value indicating the class of the gene-set, e.g. "GO".}

\item{metric_range}{optional range of the ranking metric.}
}
\value{
a random-walk plot for selected gene-set
}
\description{
Generate high-quality GSEA random-walk figures.
}
\details{
this function does not support output from GSEA \href{https://github.com/GSEA-MSigDB/GSEA_R}{R implementation}. Scripts initially implemented by Thomas Kuilman.
}
